@@ -1,5 +1,4 @@
-#
-#  Copyright 2009-2013 MongoDB, Inc.
+#  Copyright 2009 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,9 +11,8 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
-use 5.008;
+use 5.010001;
 use strict;
 use warnings;
 
@@ -22,29 +20,21 @@ package MongoDB;
 # ABSTRACT: Official MongoDB Driver for Perl
 
 use version;
-our $VERSION = 'v1.5.0'; # TRIAL
+our $VERSION = 'v1.999.0'; # TRIAL
 
 # regexp_pattern was unavailable before 5.10, had to be exported to load the
 # function implementation on 5.10, and was automatically available in 5.10.1
 use if ($] eq '5.010000'), 're', 'regexp_pattern';
 
 use Carp ();
-use MongoDB::BSON;
 use MongoDB::MongoClient;
 use MongoDB::Database;
 use MongoDB::Collection;
-use MongoDB::DBRef;
-use MongoDB::OID;
-use MongoDB::Timestamp;
-use MongoDB::BSON::Binary;
-use MongoDB::BSON::Regexp;
 use MongoDB::BulkWrite;
 use MongoDB::_Link;
 use MongoDB::_Protocol;
 use BSON::Types;
 
-*read_documents = \&MongoDB::BSON::decode_bson;
-
 # regexp_pattern was unavailable before 5.10, had to be exported to load the
 # function implementation on 5.10, and was automatically available in 5.10.1
 if ( $] eq '5.010' ) {
@@ -81,20 +71,6 @@ sub connect {
     return MongoDB::MongoClient->new( $options );
 }
 
-sub force_double {
-    if ( ref $_[0] ) {
-        Carp::croak("Can't force a reference into a double");
-    }
-    return $_[0] = unpack("d",pack("d", $_[0]));
-}
-
-sub force_int {
-    if ( ref $_[0] ) {
-        Carp::croak("Can't force a reference into an int");
-    }
-    return $_[0] = int($_[0]);
-}
-
 1;
 
 =pod
@@ -107,12 +83,46 @@ MongoDB - Official MongoDB Driver for Perl
 
 =head1 VERSION
 
-version v1.5.0
+version v1.999.0
 
-B<NOTE:> The v1.5.x versions are development releases in advance of the
-MongoDB Perl Driver v1.6.0. They are available for evaluation and testing
+This is the Alpha 1 release for v2.0.0.
+
+=head1 ALPHA RELEASE
+
+B<NOTE:> The v1.999.x versions are development releases in advance of the
+MongoDB Perl Driver v2.0.0. They are available for evaluation and testing
 and should not be used in production.
 
+Details on incompatible changes and how to upgrade applications may be
+found in L<MongoDB::Upgrading>.
+
+This development release includes several new features, including:
+
+=over
+
+=item *
+
+BSON encoding and decoding uses the external L<BSON> and optional L<BSON::XS>
+libraries
+
+=item *
+
+Support for MongoDB 3.6 features, including sessions and retryable writes
+
+=item *
+
+Wire protocol compression
+
+=item *
+
+Command monitoring callback support
+
+=back
+
+B<NOTE:> Some MongoDB 4.0 features are still under development and are not
+yet included in the driver, including transactions and
+database/cluster-wide change streams.
+
 =head1 SYNOPSIS
 
     use MongoDB;
@@ -214,9 +224,12 @@ For example, if the set name is "setA":
 
     $client = MongoDB->connect("mongodb://example.com/?replicaSet=setA");
 
-=for Pod::Coverage force_double
-force_int
-read_documents
+=begin Pod::Coverage
+
+
+
+
+=end Pod::Coverage
 
 =head1 SEMANTIC VERSIONING SCHEME
 
@@ -257,8 +270,7 @@ deprecation warning is issued once per call-site for deprecated methods.)
 
 =head1 THREADS
 
-Because of well-known bugs, use of threads on perls before v5.8.5 is
-not supported.
+Per L<threads> documentation, use of Perl threads is discouraged.
 
 =for :stopwords cpan testmatrix url annocpan anno bugtracker rt cpants kwalitee diff irc mailto metadata placeholders metacpan
 
@@ -303,7 +315,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 CONTRIBUTORS
 
-=for stopwords Andrew Page Andrey Khozov Ashley Willis Ask Bjørn Hansen Bernard Gorman Brendan W. McAdams Casey Rojas Christian Sturm Walde Colin Cyr Danny Raetzsch David Morrison Nadle Steinbrunner Storch diegok D. Ilmari Mannsåker Eric Daniels Gerard Goossen Glenn Fowler Graham Barr Hao Wu Jason Carey Toffaletti Johann Rolschewski Joseph Harnish Josh Matthews Joshua Juran J. Stewart Kamil Slowikowski Ken Williams Matthew Shopsin Michael Langner Rotmanov Mike Dirolf Mohammad S Anwar Nickola Trupcheff Nigel Gregoire Niko Tyni Nuno Carvalho Orlando Vazquez Othello Maurer Pan Fan Pavel Denisov Rahul Dhodapkar Robin Lee Roman Yerin Ronald J Kimball Ryan Chipman Slaven Rezic Stephen Oberholtzer Steve Sanbeg Stuart Watt Uwe Voelker Whitney Jackson Xtreak Zhihong Zhang
+=for stopwords Andrew Page Andrey Khozov Ashley Willis Ask Bjørn Hansen Bernard Gorman Brendan W. McAdams Brian Moss Casey Rojas Christian Sturm Walde Colin Cyr Danny Raetzsch David Morrison Nadle Steinbrunner Storch diegok D. Ilmari Mannsåker Eric Daniels Gerard Goossen Glenn Fowler Graham Barr Hao Wu Harish Upadhyayula Jason Carey Toffaletti Johann Rolschewski John A. Kunze Joseph Harnish Josh Matthews Joshua Juran J. Stewart Kamil Slowikowski Ken Williams Matthew Shopsin Michael Langner Rotmanov Mike Dirolf Mohammad S Anwar Nickola Trupcheff Nigel Gregoire Niko Tyni Nuno Carvalho Orlando Vazquez Othello Maurer Pan Fan Pavel Denisov Rahul Dhodapkar Robert Sedlacek (Shadowcat Systems Ltd) Robin Lee Roman Yerin Ronald J Kimball Ryan Chipman Slaven Rezic Stephen Oberholtzer Steve Sanbeg Stuart Watt Thomas Bloor Uwe Voelker Whitney Jackson Xavier Guimard Xtreak Zhihong Zhang
 
 =over 4
 
@@ -333,6 +345,10 @@ Brendan W. McAdams <brendan@mongodb.com>
 
 =item *
 
+Brian Moss <kallimachos@gmail.com>
+
+=item *
+
 Casey Rojas <casey.j.rojas@gmail.com>
 
 =item *
@@ -401,6 +417,10 @@ Hao Wu <echowuhao@gmail.com>
 
 =item *
 
+Harish Upadhyayula <hupadhyayula@dealersocket.com>
+
+=item *
+
 Jason Carey <jason.carey@mongodb.com>
 
 =item *
@@ -413,6 +433,10 @@ Johann Rolschewski <rolschewski@gmail.com>
 
 =item *
 
+John A. Kunze <jak@ucop.edu>
+
+=item *
+
 Joseph Harnish <bigjoe1008@gmail.com>
 
 =item *
@@ -493,6 +517,10 @@ Rahul Dhodapkar <rahul@mongodb.com>
 
 =item *
 
+Robert Sedlacek (Shadowcat Systems Ltd) <phaylon@cpan.org>
+
+=item *
+
 Robin Lee <cheeselee@fedoraproject.org>
 
 =item *
@@ -525,6 +553,10 @@ Stuart Watt <stuart@morungos.com>
 
 =item *
 
+Thomas Bloor (Shadowcat Systems Ltd) <tbsliver@cpan.org>
+
+=item *
+
 Uwe Voelker <uwe.voelker@xing.com>
 
 =item *
@@ -533,6 +565,10 @@ Whitney Jackson <whjackson@gmail.com>
 
 =item *
 
+Xavier Guimard <x.guimard@free.fr>
+
+=item *
+
 Xtreak <tirkarthi@users.noreply.github.com>
 
 =item *
@@ -543,7 +579,7 @@ Zhihong Zhang <zzh_621@yahoo.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2016 by MongoDB, Inc.
+This software is Copyright (c) 2018 by MongoDB, Inc.
 
 This is free software, licensed under:
 
