@@ -1,5 +1,4 @@
-#
-#  Copyright 2014 MongoDB, Inc.
+#  Copyright 2014 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,14 +11,15 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
+use strict;
+use warnings;
 package MongoDB::CommandResult;
 
 # ABSTRACT: MongoDB generic command result document
 
 use version;
-our $VERSION = 'v1.5.0'; # TRIAL
+our $VERSION = 'v1.999.0'; # TRIAL
 
 use Moo;
 use MongoDB::Error;
@@ -145,15 +145,6 @@ sub assert_no_write_concern_error {
     return 1;
 }
 
-# deprecated
-sub result {
-    my $self = shift;
-
-    $self->_warn_deprecated( 'result' => [qw/output/] );
-
-    return $self->output;
-}
-
 1;
 
 __END__
@@ -168,7 +159,7 @@ MongoDB::CommandResult - MongoDB generic command result document
 
 =head1 VERSION
 
-version v1.5.0
+version v1.999.0
 
 =head1 DESCRIPTION
 
@@ -209,19 +200,6 @@ Throws an exception if a write concern error occurred
 
 =for Pod::Coverage result
 
-=head1 DEPRECATIONS
-
-The methods still exist, but are no longer documented.  In a future version
-they will warn when used, then will eventually be removed.
-
-=over 4
-
-=item *
-
-result
-
-=back
-
 =head1 AUTHORS
 
 =over 4
@@ -250,7 +228,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2016 by MongoDB, Inc.
+This software is Copyright (c) 2018 by MongoDB, Inc.
 
 This is free software, licensed under:
 
