@@ -1,5 +1,4 @@
-#
-#  Copyright 2009-2013 MongoDB, Inc.
+#  Copyright 2009 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,19 +11,19 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
+use strict;
+use warnings;
 package MongoDB::Database;
 
 
 # ABSTRACT: A MongoDB Database
 
 use version;
-our $VERSION = 'v1.5.0'; # TRIAL
+our $VERSION = 'v1.999.0'; # TRIAL
 
 use MongoDB::CommandResult;
 use MongoDB::Error;
-use MongoDB::GridFS;
 use MongoDB::GridFSBucket;
 use MongoDB::Op::_Command;
 use MongoDB::Op::_DropDatabase;
@@ -36,6 +35,7 @@ use MongoDB::_Types qw(
     ReadPreference
     ReadConcern
     WriteConcern
+    is_OrderedDoc
 );
 use Types::Standard qw(
     InstanceOf
@@ -132,8 +132,8 @@ has max_time_ms => (
 #pod =attr bson_codec
 #pod
 #pod An object that provides the C<encode_one> and C<decode_one> methods, such as
-#pod from L<MongoDB::BSON>.  It may be initialized with a hash reference that will
-#pod be coerced into a new MongoDB::BSON object.  By default it will be inherited
+#pod from L<BSON>.  It may be initialized with a hash reference that will
+#pod be coerced into a new L<BSON> object.  By default it will be inherited
 #pod from a L<MongoDB::MongoClient> object.
 #pod
 #pod =cut
@@ -145,10 +145,6 @@ has bson_codec => (
     required => 1,
 );
 
-with $_ for qw(
-  MongoDB::Role::_DeprecationWarner
-);
-
 #--------------------------------------------------------------------------#
 # methods
 #--------------------------------------------------------------------------#
@@ -183,6 +179,8 @@ with $_ for qw(
 #pod * C<batchSize> – the number of documents to return per batch.
 #pod * C<maxTimeMS> – the maximum amount of time in milliseconds to allow the
 #pod   command to run.  (Note, this will be ignored for servers before version 2.6.)
+#pod * C<session> - the session to use for these operations. If not supplied, will
+#pod   use an implicit session. For more information see L<MongoDB::ClientSession>
 #pod
 #pod =cut
 
@@ -198,12 +196,16 @@ sub list_collections {
         $options->{maxTimeMS} = $self->max_time_ms;
     }
 
+    my $session = $self->_get_session_from_hashref( $options );
+
     my $op = MongoDB::Op::_ListCollections->_new(
-        db_name    => $self->name,
-        client     => $self->_client,
-        bson_codec => $self->bson_codec,
-        filter     => $filter,
-        options    => $options,
+        db_name             => $self->name,
+        client              => $self->_client,
+        bson_codec          => $self->bson_codec,
+        filter              => $filter,
+        options             => $options,
+        session             => $session,
+        monitoring_callback => $self->_client->monitoring_callback,
     );
 
     return $self->_client->send_primary_op($op);
@@ -222,6 +224,14 @@ sub list_collections {
 #pod documentation|http://docs.mongodb.org/manual/reference/command/listCollections/>
 #pod for more details on filtering for specific collections.
 #pod
+#pod A hashref of options may also be provided.
+#pod
+#pod Valid options include:
+#pod
+#pod =for :list
+#pod * C<session> - the session to use for these operations. If not supplied, will
+#pod   use an implicit session. For more information see L<MongoDB::ClientSession>
+#pod
 #pod B<Warning:> if the number of collections is very large, this may return
 #pod a very large result.  Either pass an appropriate filter, or use
 #pod L</list_collections> to iterate over collections instead.
@@ -229,18 +239,9 @@ sub list_collections {
 #pod =cut
 
 sub collection_names {
-    my ( $self, $filter ) = @_;
-    $filter ||= {};
+    my $self = shift;
 
-    my $op = MongoDB::Op::_ListCollections->_new(
-        db_name    => $self->name,
-        client     => $self->_client,
-        bson_codec => $self->bson_codec,
-        filter     => $filter,
-        options    => {},
-    );
-
-    my $res = $self->_client->send_primary_op($op);
+    my $res = $self->list_collections( @_ );
 
     return map { $_->{name} } $res->all;
 }
@@ -313,60 +314,35 @@ sub get_gridfsbucket {
 
 { no warnings 'once'; *gfs = \&get_gridfsbucket }
 
-#pod =method get_gridfs (DEPRECATED)
-#pod
-#pod     my $grid = $database->get_gridfs;
-#pod     my $grid = $database->get_gridfs("fs");
-#pod     my $grid = $database->get_gridfs("fs", $options);
-#pod
-#pod The L<MongoDB::GridFS> class has been deprecated in favor of the new MongoDB
-#pod driver-wide standard GridFS API, available via L<MongoDB::GridFSBucket> and
-#pod the C<get_gridfsbucket>/C<gfs> methods.
-#pod
-#pod This method returns a L<MongoDB::GridFS> for storing and retrieving files
-#pod from the database.  Default prefix is "fs", making C<$grid-E<gt>files>
-#pod "fs.files" and C<$grid-E<gt>chunks> "fs.chunks".
-#pod
-#pod It takes an optional hash reference of options that are passed to the
-#pod L<MongoDB::GridFS> constructor.
-#pod
-#pod See L<MongoDB::GridFS> for more information.
-#pod
-#pod =cut
-
-sub get_gridfs {
-    my ($self, $prefix, $options) = @_;
-    $prefix = "fs" unless $prefix;
-
-    $self->_warn_deprecated( 'get_gridfs' => [qw/get_gridfsbucket gfs/] );
-
-    return MongoDB::GridFS->new(
-        read_preference => $self->read_preference,
-        write_concern   => $self->write_concern,
-        max_time_ms     => $self->max_time_ms,
-        bson_codec      => $self->bson_codec,
-        ( $options ? %$options : () ),
-        # not allowed to be overridden by options
-        _database => $self,
-        prefix => $prefix
-    );
-}
-
 #pod =method drop
 #pod
 #pod     $database->drop;
 #pod
 #pod Deletes the database.
 #pod
+#pod A hashref of options may also be provided.
+#pod
+#pod Valid options include:
+#pod
+#pod =for :list
+#pod * C<session> - the session to use for these operations. If not supplied, will
+#pod   use an implicit session. For more information see L<MongoDB::ClientSession>
+#pod
 #pod =cut
 
 sub drop {
-    my ($self) = @_;
+    my ( $self, $options ) = @_;
+
+    my $session = $self->_get_session_from_hashref( $options );
+
     return $self->_client->send_write_op(
         MongoDB::Op::_DropDatabase->_new(
-            db_name       => $self->name,
-            bson_codec    => $self->bson_codec,
-            write_concern => $self->write_concern,
+            client              => $self->_client,
+            db_name             => $self->name,
+            bson_codec          => $self->bson_codec,
+            write_concern       => $self->write_concern,
+            session             => $session,
+            monitoring_callback => $self->_client->monitoring_callback,
         )
     )->output;
 }
@@ -380,17 +356,31 @@ sub drop {
 #pod         { mode => 'secondaryPreferred' }
 #pod     );
 #pod
+#pod     my $output = $database->run_command(
+#pod         [ some_command => 1 ],
+#pod         $read_preference,
+#pod         $options
+#pod     );
+#pod
 #pod This method runs a database command.  The first argument must be a document
 #pod with the command and its arguments.  It should be given as an array reference
 #pod of key-value pairs or a L<Tie::IxHash> object with the command name as the
-#pod first key.  The use of a hash reference will only reliably work for commands
-#pod without additional parameters.
+#pod first key.  An error will be thrown if the command is not an
+#pod L<ordered document|MongoDB::Collection/Ordered document>.
 #pod
 #pod By default, commands are run with a read preference of 'primary'.  An optional
 #pod second argument may specify an alternative read preference.  If given, it must
 #pod be a L<MongoDB::ReadPreference> object or a hash reference that can be used to
 #pod construct one.
 #pod
+#pod A hashref of options may also be provided.
+#pod
+#pod Valid options include:
+#pod
+#pod =for :list
+#pod * C<session> - the session to use for these operations. If not supplied, will
+#pod   use an implicit session. For more information see L<MongoDB::ClientSession>
+#pod
 #pod It returns the output of the command (a hash reference) on success or throws a
 #pod L<MongoDB::DatabaseError|MongoDB::Error/MongoDB::DatabaseError> exception if
 #pod the command fails.
@@ -406,18 +396,25 @@ sub drop {
 #pod =cut
 
 sub run_command {
-    my ( $self, $command, $read_pref ) = @_;
+    my ( $self, $command, $read_pref, $options ) = @_;
+    MongoDB::UsageError->throw("command was not an ordered document")
+       if ! is_OrderedDoc($command);
 
     $read_pref = MongoDB::ReadPreference->new(
         ref($read_pref) ? $read_pref : ( mode => $read_pref ) )
       if $read_pref && ref($read_pref) ne 'MongoDB::ReadPreference';
 
+    my $session = $self->_get_session_from_hashref( $options );
+
     my $op = MongoDB::Op::_Command->_new(
-        db_name     => $self->name,
-        query       => $command,
-        query_flags => {},
-        bson_codec  => $self->bson_codec,
-        read_preference => $read_pref,
+        client              => $self->_client,
+        db_name             => $self->name,
+        query               => $command,
+        query_flags         => {},
+        bson_codec          => $self->bson_codec,
+        read_preference     => $read_pref,
+        session             => $session,
+        monitoring_callback => $self->_client->monitoring_callback,
     );
 
     my $obj = $self->_client->send_read_op($op);
@@ -425,41 +422,67 @@ sub run_command {
     return $obj->output;
 }
 
-#--------------------------------------------------------------------------#
-# deprecated methods
-#--------------------------------------------------------------------------#
+sub _aggregate {
+    MongoDB::UsageError->throw("pipeline argument must be an array reference")
+      unless ref( $_[1] ) eq 'ARRAY';
 
-sub eval {
-    my ($self, $code, $args, $nolock) = @_;
+    my ( $self, $pipeline, $options ) = @_;
+    $options ||= {};
 
-    $self->_warn_deprecated( 'eval', "Run manually via run_command instead." );
+    my $session = $self->_get_session_from_hashref( $options );
 
-    $nolock = boolean::false unless defined $nolock;
-
-    my $cmd = tie(my %hash, 'Tie::IxHash');
-    %hash = ('$eval' => $code,
-             'args' => $args,
-             'nolock' => $nolock);
-
-    my $output = $self->run_command($cmd);
-    if (ref $output eq 'HASH' && exists $output->{'retval'}) {
-        return $output->{'retval'};
+    # boolify some options
+    for my $k (qw/allowDiskUse explain/) {
+        $options->{$k} = ( $options->{$k} ? true : false ) if exists $options->{$k};
     }
-    else {
-        return $output;
+
+    # possibly fallback to default maxTimeMS
+    if ( ! exists $options->{maxTimeMS} && $self->max_time_ms ) {
+        $options->{maxTimeMS} = $self->max_time_ms;
     }
+
+    # read preferences are ignored if the last stage is $out
+    my ($last_op) = keys %{ $pipeline->[-1] };
+
+    my $op = MongoDB::Op::_Aggregate->_new(
+        pipeline        => $pipeline,
+        options         => $options,
+        read_concern    => $self->read_concern,
+        has_out         => $last_op eq '$out',
+        client          => $self->_client,
+        bson_codec      => $self->bson_codec,
+        db_name         => $self->name,
+        coll_name       => 1,                     # Magic not-an-actual-collection number
+        full_name       => $self->name . ".1",
+        read_preference => $self->read_preference,
+        write_concern   => $self->write_concern,
+        session         => $session,
+        monitoring_callback => $self->_client->monitoring_callback,
+    );
+
+    return $self->_client->send_read_op($op);
 }
 
-sub last_error {
-    my ( $self, $opt ) = @_;
+# Extracts a session from a provided hashref, or returns an implicit session
+# Almost identical to same subroutine in Collection, however in Database the
+# client attribute is private. 
+sub _get_session_from_hashref {
+    my ( $self, $hashref ) = @_;
 
-    $self->_warn_deprecated(
-        'last_error' => "Use a write concern or manually run getlasterror with run_command." );
+    my $session = delete $hashref->{session};
 
-    return $self->run_command( [ getlasterror => 1, ( $opt ? %$opt : () ) ] );
+    if ( defined $session ) {
+        MongoDB::UsageError->throw( "Cannot use session from another client" )
+            if ( $session->client->_id ne $self->_client->_id );
+        MongoDB::UsageError->throw( "Cannot use session which has ended" )
+            if ! defined $session->session_id;
+    } else {
+        $session = $self->_client->_maybe_get_implicit_session;
+    }
+
+    return $session;
 }
 
-
 1;
 
 __END__
@@ -474,7 +497,7 @@ MongoDB::Database - A MongoDB Database
 
 =head1 VERSION
 
-version v1.5.0
+version v1.999.0
 
 =head1 SYNOPSIS
 
@@ -500,7 +523,7 @@ call C<get_database> on a L<MongoDB::MongoClient> object.
 
 =head2 Error handling
 
-Unless otherwise explictly documented, all methods throw exceptions if
+Unless otherwise explicitly documented, all methods throw exceptions if
 an error occurs.  The error types are documented in L<MongoDB::Error>.
 
 To catch and handle errors, the L<Try::Tiny> and L<Safe::Isa> modules
@@ -561,8 +584,8 @@ was when the C<$maxTimeMS> meta-operator was introduced.
 =head2 bson_codec
 
 An object that provides the C<encode_one> and C<decode_one> methods, such as
-from L<MongoDB::BSON>.  It may be initialized with a hash reference that will
-be coerced into a new MongoDB::BSON object.  By default it will be inherited
+from L<BSON>.  It may be initialized with a hash reference that will
+be coerced into a new L<BSON> object.  By default it will be inherited
 from a L<MongoDB::MongoClient> object.
 
 =head1 METHODS
@@ -603,6 +626,10 @@ C<batchSize> – the number of documents to return per batch.
 
 C<maxTimeMS> – the maximum amount of time in milliseconds to allow the command to run.  (Note, this will be ignored for servers before version 2.6.)
 
+=item *
+
+C<session> - the session to use for these operations. If not supplied, will use an implicit session. For more information see L<MongoDB::ClientSession>
+
 =back
 
 =head2 collection_names
@@ -618,6 +645,18 @@ L<listCollections command
 documentation|http://docs.mongodb.org/manual/reference/command/listCollections/>
 for more details on filtering for specific collections.
 
+A hashref of options may also be provided.
+
+Valid options include:
+
+=over 4
+
+=item *
+
+C<session> - the session to use for these operations. If not supplied, will use an implicit session. For more information see L<MongoDB::ClientSession>
+
+=back
+
 B<Warning:> if the number of collections is very large, this may return
 a very large result.  Either pass an appropriate filter, or use
 L</list_collections> to iterate over collections instead.
@@ -652,31 +691,24 @@ See L<MongoDB::GridFSBucket> for more information.
 
 The C<gfs> method is an alias for C<get_gridfsbucket>.
 
-=head2 get_gridfs (DEPRECATED)
-
-    my $grid = $database->get_gridfs;
-    my $grid = $database->get_gridfs("fs");
-    my $grid = $database->get_gridfs("fs", $options);
-
-The L<MongoDB::GridFS> class has been deprecated in favor of the new MongoDB
-driver-wide standard GridFS API, available via L<MongoDB::GridFSBucket> and
-the C<get_gridfsbucket>/C<gfs> methods.
-
-This method returns a L<MongoDB::GridFS> for storing and retrieving files
-from the database.  Default prefix is "fs", making C<$grid-E<gt>files>
-"fs.files" and C<$grid-E<gt>chunks> "fs.chunks".
-
-It takes an optional hash reference of options that are passed to the
-L<MongoDB::GridFS> constructor.
-
-See L<MongoDB::GridFS> for more information.
-
 =head2 drop
 
     $database->drop;
 
 Deletes the database.
 
+A hashref of options may also be provided.
+
+Valid options include:
+
+=over 4
+
+=item *
+
+C<session> - the session to use for these operations. If not supplied, will use an implicit session. For more information see L<MongoDB::ClientSession>
+
+=back
+
 =head2 run_command
 
     my $output = $database->run_command([ some_command => 1 ]);
@@ -686,17 +718,35 @@ Deletes the database.
         { mode => 'secondaryPreferred' }
     );
 
+    my $output = $database->run_command(
+        [ some_command => 1 ],
+        $read_preference,
+        $options
+    );
+
 This method runs a database command.  The first argument must be a document
 with the command and its arguments.  It should be given as an array reference
 of key-value pairs or a L<Tie::IxHash> object with the command name as the
-first key.  The use of a hash reference will only reliably work for commands
-without additional parameters.
+first key.  An error will be thrown if the command is not an
+L<ordered document|MongoDB::Collection/Ordered document>.
 
 By default, commands are run with a read preference of 'primary'.  An optional
 second argument may specify an alternative read preference.  If given, it must
 be a L<MongoDB::ReadPreference> object or a hash reference that can be used to
 construct one.
 
+A hashref of options may also be provided.
+
+Valid options include:
+
+=over 4
+
+=item *
+
+C<session> - the session to use for these operations. If not supplied, will use an implicit session. For more information see L<MongoDB::ClientSession>
+
+=back
+
 It returns the output of the command (a hash reference) on success or throws a
 L<MongoDB::DatabaseError|MongoDB::Error/MongoDB::DatabaseError> exception if
 the command fails.
@@ -711,19 +761,6 @@ on database commands: L<http://dochub.mongodb.org/core/commands>.
 
 =for Pod::Coverage last_error
 
-=head1 DEPRECATIONS
-
-The methods still exist, but are no longer documented.  In a future version
-they will warn when used, then will eventually be removed.
-
-=over 4
-
-=item *
-
-last_error
-
-=back
-
 =head1 AUTHORS
 
 =over 4
@@ -752,7 +789,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2016 by MongoDB, Inc.
+This software is Copyright (c) 2018 by MongoDB, Inc.
 
 This is free software, licensed under:
 
