@@ -1,5 +1,4 @@
-#
-#  Copyright 2014 MongoDB, Inc.
+#  Copyright 2014 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,19 +11,20 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
+use strict;
+use warnings;
 package MongoDB::DeleteResult;
 
 # ABSTRACT: MongoDB deletion result object
 
 use version;
-our $VERSION = 'v1.5.0'; # TRIAL
+our $VERSION = 'v1.999.0'; # TRIAL
 
 use Moo;
 use MongoDB::_Constants;
-use Types::Standard qw(
-    Num
+use MongoDB::_Types qw(
+    Numish
 );
 use namespace::clean;
 
@@ -42,7 +42,7 @@ with $_ for qw(
 has deleted_count => (
     is      => 'ro',
     default => 0,
-    isa => Num,
+    isa => Numish,
 );
 
 1;
@@ -59,7 +59,7 @@ MongoDB::DeleteResult - MongoDB deletion result object
 
 =head1 VERSION
 
-version v1.5.0
+version v1.999.0
 
 =head1 SYNOPSIS
 
@@ -129,7 +129,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2016 by MongoDB, Inc.
+This software is Copyright (c) 2018 by MongoDB, Inc.
 
 This is free software, licensed under:
 
