@@ -1,5 +1,4 @@
-#
-#  Copyright 2014 MongoDB, Inc.
+#  Copyright 2014 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,8 +11,7 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
-use 5.008;
+
 use strict;
 use warnings;
 
@@ -25,7 +23,7 @@ package MongoDB::Error;
 
 use version;
 
-our $VERSION = 'v1.5.0'; # TRIAL
+our $VERSION = 'v1.999.0'; # TRIAL
 
 use Moo;
 use Carp;
@@ -43,7 +41,10 @@ BEGIN {
     $ERROR_CODES = {
         BAD_VALUE                 => 2,
         UNKNOWN_ERROR             => 8,
+        USER_NOT_FOUND            => 11,
         NAMESPACE_NOT_FOUND       => 26,
+        INDEX_NOT_FOUND           => 27,
+        CURSOR_NOT_FOUND          => 43,
         EXCEEDED_TIME_LIMIT       => 50,
         COMMAND_NOT_FOUND         => 59,
         WRITE_CONCERN_ERROR       => 64,
@@ -108,13 +109,19 @@ sub throw {
   die $throwable;
 }
 
+# internal flag indicating if an operation should be retried when
+# an error occurs.
+sub _is_resumable { 1 }
+
 #--------------------------------------------------------------------------#
 # Subclasses with attributes included inline below
 #--------------------------------------------------------------------------#
 
 package MongoDB::DatabaseError;
 use Moo;
-use Types::Standard qw(Num);
+use MongoDB::_Types qw(
+    Numish
+);
 use namespace::clean;
 
 extends("MongoDB::Error");
@@ -127,12 +134,14 @@ has result => (
 
 has code => (
     is      => 'ro',
-    isa     => Num,
+    isa     => Numish,
     builder => '_build_code',
 );
 
 sub _build_code { return MongoDB::Error::UNKNOWN_ERROR() }
 
+sub _is_resumable { 0 }
+
 package MongoDB::DocumentError;
 
 use Moo;
@@ -190,6 +199,8 @@ use Moo;
 use namespace::clean;
 extends 'MongoDB::Error';
 
+sub _is_resumable { 1 }
+
 package MongoDB::HandshakeError;
 use Moo;
 use namespace::clean;
@@ -228,6 +239,7 @@ use Moo;
 use namespace::clean;
 extends 'MongoDB::DatabaseError';
 sub _build_code { return MongoDB::Error::NOT_MASTER() }
+sub _is_resumable { 1 }
 
 package MongoDB::WriteError;
 use Moo;
@@ -246,11 +258,18 @@ use Moo;
 use namespace::clean;
 extends 'MongoDB::Error';
 
-package MongoDB::CursorNotFoundError;
+package MongoDB::ConfigurationError;
 use Moo;
 use namespace::clean;
 extends 'MongoDB::Error';
 
+package MongoDB::CursorNotFoundError;
+use Moo;
+use namespace::clean;
+extends 'MongoDB::DatabaseError';
+sub _build_code { return MongoDB::Error::CURSOR_NOT_FOUND() }
+sub _is_resumable { 1 }
+
 package MongoDB::DecodingError;
 use Moo;
 use namespace::clean;
@@ -276,19 +295,26 @@ use Moo;
 use namespace::clean;
 extends 'MongoDB::Error';
 
+package MongoDB::InvalidOperationError;
+use Moo;
+use namespace::clean;
+extends 'MongoDB::Error';
+
 #--------------------------------------------------------------------------#
 # Private error classes
 #--------------------------------------------------------------------------#
 package MongoDB::_CommandSizeError;
 use Moo;
-use Types::Standard qw(Int);
+use MongoDB::_Types qw(
+    Intish
+);
 use namespace::clean;
 
 extends("MongoDB::Error");
 
 has size => (
     is       => 'ro',
-    isa      => Int,
+    isa      => Intish,
     required => 1,
 );
 
@@ -304,7 +330,7 @@ MongoDB::Error - MongoDB Driver Error classes
 
 =head1 VERSION
 
-version v1.5.0
+version v1.999.0
 
 =head1 SYNOPSIS
 
@@ -317,11 +343,11 @@ version v1.5.0
 
 =head1 DESCRIPTION
 
-This class defines a heirarchy of exception objects.
+This class defines a hierarchy of exception objects.
 
 =head1 USAGE
 
-Unless otherwise explictly documented, all driver methods throw exceptions if
+Unless otherwise explicitly documented, all driver methods throw exceptions if
 an error occurs.
 
 To catch and handle errors, the L<Try::Tiny> and L<Safe::Isa> modules
@@ -356,10 +382,12 @@ To retry failures automatically, consider using L<Try::Tiny::Retry>.
         |   |
         |   |->MongoDB::NetworkError
         |
-        |->MongoDB::CursorNotFoundError
+        |->MongoDB::ConfigurationError
         |
         |->MongoDB::DatabaseError
         |   |
+        |   |->MongoDB::CursorNotFoundError
+        |   |
         |   |->MongoDB::DuplicateKeyError
         |   |
         |   |->MongoDB::NotMasterError
@@ -376,6 +404,8 @@ To retry failures automatically, consider using L<Try::Tiny::Retry>.
         |
         |->MongoDB::InternalError
         |
+        |->MongoDB::InvalidOperationError
+        |
         |->MongoDB::ProtocolError
         |
         |->MongoDB::SelectionError
@@ -419,6 +449,12 @@ handshakes fail.
 This error is thrown when a socket error occurs, when the wrong number of bytes
 are read, or other wire-related errors occur.
 
+=head2 MongoDB::ConfigurationError
+
+This error is thrown when there is a configuration error between the MongoDB
+deployment and the configuration of the client, such as when trying to use
+explicit sessions on a MongoDB < 3.6
+
 =head2 MongoDB::CursorNotFoundError
 
 This error indicates that a cursor timed out on a server.
@@ -590,7 +626,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2016 by MongoDB, Inc.
+This software is Copyright (c) 2018 by MongoDB, Inc.
 
 This is free software, licensed under:
 
