@@ -1,5 +1,4 @@
-#
-#  Copyright 2009-2015 MongoDB, Inc.
+#  Copyright 2015 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -13,17 +12,20 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 
+use strict;
+use warnings;
 package MongoDB::GridFSBucket;
 
 # ABSTRACT: A file storage abstraction
 
 use version;
-our $VERSION = 'v1.5.0'; # TRIAL
+our $VERSION = 'v1.999.0'; # TRIAL
 
 use Moo;
 use MongoDB::GridFSBucket::DownloadStream;
 use MongoDB::GridFSBucket::UploadStream;
 use MongoDB::_Types qw(
+  Boolish
   ReadPreference
   WriteConcern
   ReadConcern
@@ -32,8 +34,6 @@ use MongoDB::_Types qw(
 );
 use Scalar::Util qw/reftype/;
 use Types::Standard qw(
-  Bool
-  Int
   Str
   InstanceOf
 );
@@ -133,8 +133,8 @@ has read_preference => (
 #pod =attr bson_codec
 #pod
 #pod An object that provides the C<encode_one> and C<decode_one> methods, such
-#pod as from L<MongoDB::BSON>.  It may be initialized with a hash reference that
-#pod will be coerced into a new MongoDB::BSON object.  By default it will be
+#pod as from L<BSON>.  It may be initialized with a hash reference that
+#pod will be coerced into a new BSON object.  By default it will be
 #pod inherited from a L<MongoDB::Database> object.
 #pod
 #pod =cut
@@ -166,7 +166,7 @@ has max_time_ms => (
 # determines whether or not to attempt index creation
 has _tried_indexing => (
     is => 'rwp',
-    isa => Bool,
+    isa => Boolish,
 );
 
 has _files => (
@@ -220,7 +220,7 @@ sub _create_indexes {
 
     my $pf = $self->_files->clone( read_preference => 'primary' );
 
-    return if $pf->count > 0;
+    return if $pf->count_documents({}) > 0;
 
     my $pfi = $pf->indexes;
     my $pci = $self->_chunks->clone( read_preference => 'primary' )->indexes;
@@ -393,8 +393,9 @@ sub open_upload_stream {
 
 sub open_upload_stream_with_id {
     my ( $self, $id, $filename, $options ) = @_;
+    my $id_copy = $id;
     MongoDB::UsageError->throw('No id provided to open_upload_stream_with_id')
-      unless defined $id && length $id;
+      unless defined $id_copy && length $id_copy;
     MongoDB::UsageError->throw('No filename provided to open_upload_stream_with_id')
       unless defined $filename && length $filename;
 
@@ -500,8 +501,9 @@ sub upload_from_stream {
 
 sub upload_from_stream_with_id {
     my ( $self, $id, $filename, $source_fh, $options ) = @_;
+    my $id_copy = $id; # preserve number/string form
     MongoDB::UsageError->throw('No id provided to upload_from_stream_with_id')
-      unless defined $id && length $id;
+      unless defined $id_copy && length $id_copy;
     MongoDB::UsageError->throw('No filename provided to upload_from_stream_with_id')
       unless defined $filename && length $filename;
     MongoDB::UsageError->throw('No handle provided to upload_from_stream_with_id')
@@ -571,7 +573,7 @@ MongoDB::GridFSBucket - A file storage abstraction
 
 =head1 VERSION
 
-version v1.5.0
+version v1.999.0
 
 =head1 SYNOPSIS
 
@@ -583,7 +585,7 @@ version v1.5.0
     $stream->close;
 
     # find and download a file
-    $result  = $bucket-find({filename => "foo.txt"});
+    $result  = $bucket->find({filename => "foo.txt"});
     $file_id = $result->next->{_id};
     $stream  = $bucket->open_download_stream($file_id)
     $data    = do { local $/; $stream->readline() };
@@ -613,7 +615,7 @@ Valid file documents typically include the following fields:
 
 =item *
 
-_id – a unique ID for this document, typically type BSON ObjectId.
+_id – a unique ID for this document, typically a BSON ObjectId.
 
 =item *
 
@@ -633,7 +635,7 @@ md5 – a hash of the contents of the stored file
 
 =item *
 
-filename – the name of this stored file; this does not need to be unique
+filename – the name of this stored file; the combination of filename and uploadDate (millisecond resolution) must be unique
 
 =item *
 
@@ -672,7 +674,7 @@ streaming: you provide a file handle to read from (upload) or print to (download
 
 =head2 Error handling
 
-Unless otherwise explictly documented, all methods throw exceptions if
+Unless otherwise explicitly documented, all methods throw exceptions if
 an error occurs.  The error types are documented in L<MongoDB::Error>.
 
 =head1 ATTRIBUTES
@@ -720,8 +722,8 @@ if all file and chunk documents have not replicated to all secondaries.
 =head2 bson_codec
 
 An object that provides the C<encode_one> and C<decode_one> methods, such
-as from L<MongoDB::BSON>.  It may be initialized with a hash reference that
-will be coerced into a new MongoDB::BSON object.  By default it will be
+as from L<BSON>.  It may be initialized with a hash reference that
+will be coerced into a new BSON object.  By default it will be
 inherited from a L<MongoDB::Database> object.
 
 =head2 max_time_ms
@@ -919,7 +921,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2016 by MongoDB, Inc.
+This software is Copyright (c) 2018 by MongoDB, Inc.
 
 This is free software, licensed under:
 
