@@ -1,5 +1,4 @@
-#
-#  Copyright 2009-2015 MongoDB, Inc.
+#  Copyright 2015 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -13,23 +12,25 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 
+use strict;
+use warnings;
 package MongoDB::GridFSBucket::DownloadStream;
 
 # ABSTRACT: File handle abstraction for downloading
 
 use version;
-our $VERSION = 'v1.5.0'; # TRIAL
+our $VERSION = 'v1.999.0'; # TRIAL
 
 use Moo;
 use Types::Standard qw(
   Str
-  Bool
   Maybe
   HashRef
   InstanceOf
   FileHandle
 );
 use MongoDB::_Types qw(
+  Boolish
   NonNegNum
 );
 use List::Util qw(max min);
@@ -42,7 +43,7 @@ use namespace::clean -except => 'meta';
 #pod Valid file documents typically include the following fields:
 #pod
 #pod =for :list
-#pod * _id – a unique ID for this document, typically a L<MongoDB::OID> object.
+#pod * _id – a unique ID for this document, typically a L<BSON::OID> object.
 #pod   Legacy GridFS files may store this value as a different type.
 #pod * length – the length of this stored file, in bytes
 #pod * chunkSize – the size, in bytes, of each full data chunk of this file.
@@ -90,7 +91,7 @@ has _offset => (
 
 has _closed => (
     is      => 'rwp',
-    isa     => Bool,
+    isa     => Boolish,
     default => 0,
 );
 
@@ -436,7 +437,7 @@ MongoDB::GridFSBucket::DownloadStream - File handle abstraction for downloading
 
 =head1 VERSION
 
-version v1.5.0
+version v1.999.0
 
 =head1 SYNOPSIS
 
@@ -470,7 +471,7 @@ Valid file documents typically include the following fields:
 
 =item *
 
-_id – a unique ID for this document, typically a L<MongoDB::OID> object. Legacy GridFS files may store this value as a different type.
+_id – a unique ID for this document, typically a L<BSON::OID> object. Legacy GridFS files may store this value as a different type.
 
 =item *
 
@@ -615,7 +616,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2016 by MongoDB, Inc.
+This software is Copyright (c) 2018 by MongoDB, Inc.
 
 This is free software, licensed under:
 
