@@ -1,5 +1,4 @@
-#
-#  Copyright 2009-2015 MongoDB, Inc.
+#  Copyright 2015 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -13,28 +12,31 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 
+use strict;
+use warnings;
 package MongoDB::GridFSBucket::UploadStream;
 
 # ABSTRACT: File handle abstraction for uploading
 
 use version;
-our $VERSION = 'v1.5.0'; # TRIAL
+our $VERSION = 'v1.999.0'; # TRIAL
 
 use Moo;
+use BSON::Bytes;
+use BSON::OID;
+use BSON::Time;
 use Encode;
-use DateTime;
 use MongoDB::Error;
-use MongoDB::OID;
-use MongoDB::BSON::Binary;
+use Time::HiRes qw/time/;
 use Types::Standard qw(
   Str
-  Bool
   Maybe
   HashRef
   ArrayRef
   InstanceOf
 );
 use MongoDB::_Types qw(
+  Boolish
   NonNegNum
 );
 use MongoDB::_Constants;
@@ -131,7 +133,7 @@ has _bucket => (
 #pod The id of the file created by the stream.  It will be stored in the C<_id>
 #pod field of the file document on a successful upload.  Some upload methods
 #pod require specifying an id at upload time.  Defaults to a newly-generated
-#pod L<MongoDB::OID>.
+#pod L<BSON::OID> or BSON codec specific equivalent.
 #pod
 #pod =cut
 
@@ -140,12 +142,14 @@ has id => (
 );
 
 sub _build_id {
-    return MongoDB::OID->new;
+    my $self = shift;
+    my $creator = $self->_bucket->bson_codec->can("create_oid");
+    return $creator ? $creator->() : BSON::OID->new();
 }
 
 has _closed => (
     is      => 'rwp',
-    isa     => Bool,
+    isa     => Boolish,
     default => 0,
 );
 
@@ -231,7 +235,7 @@ sub _flush_chunks {
           {
             files_id => $self->id,
             n        => int( $self->_current_chunk_n ),
-            data     => MongoDB::BSON::Binary->new( { data => $data } ),
+            data     => BSON::Bytes->new( data => $data ),
           };
         $self->{_current_chunk_n} += 1;
     }
@@ -304,7 +308,7 @@ sub close {
         _id        => $self->id,
         length     => $self->_length,
         chunkSize  => $self->chunk_size_bytes,
-        uploadDate => DateTime->now,
+        uploadDate => BSON::Time->new(),
         md5        => $self->_md5->hexdigest,
         filename   => $self->filename,
     };
@@ -452,7 +456,7 @@ MongoDB::GridFSBucket::UploadStream - File handle abstraction for uploading
 
 =head1 VERSION
 
-version v1.5.0
+version v1.999.0
 
 =head1 SYNOPSIS
 
@@ -527,7 +531,7 @@ document on a successful upload.
 The id of the file created by the stream.  It will be stored in the C<_id>
 field of the file document on a successful upload.  Some upload methods
 require specifying an id at upload time.  Defaults to a newly-generated
-L<MongoDB::OID>.
+L<BSON::OID> or BSON codec specific equivalent.
 
 =head2 fh
 
@@ -657,7 +661,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2016 by MongoDB, Inc.
+This software is Copyright (c) 2018 by MongoDB, Inc.
 
 This is free software, licensed under:
 
