@@ -1,5 +1,4 @@
-#
-#  Copyright 2014 MongoDB, Inc.
+#  Copyright 2014 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,23 +11,24 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
+use strict;
+use warnings;
 package MongoDB::InsertManyResult;
 
 # ABSTRACT: MongoDB single insert result object
 
 use version;
-our $VERSION = 'v1.5.0'; # TRIAL
+our $VERSION = 'v1.999.0'; # TRIAL
 
 use Moo;
 use MongoDB::_Constants;
 use MongoDB::_Types qw(
     ArrayOfHashRef
+    Numish
 );
 use Types::Standard qw(
     HashRef
-    Num
 );
 use namespace::clean;
 
@@ -46,7 +46,7 @@ with $_ for qw(
 has inserted_count => (
     is      => 'lazy',
     builder => '_build_inserted_count',
-    isa => Num,
+    isa => Numish,
 );
 
 sub _build_inserted_count
@@ -100,7 +100,7 @@ MongoDB::InsertManyResult - MongoDB single insert result object
 
 =head1 VERSION
 
-version v1.5.0
+version v1.999.0
 
 =head1 SYNOPSIS
 
@@ -180,7 +180,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2016 by MongoDB, Inc.
+This software is Copyright (c) 2018 by MongoDB, Inc.
 
 This is free software, licensed under:
 
