@@ -1,5 +1,4 @@
-#
-#  Copyright 2014 MongoDB, Inc.
+#  Copyright 2015 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,20 +11,22 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
+use strict;
+use warnings;
 package MongoDB::Op::_Count;
 
 # Encapsulate code path for count commands
 
 use version;
-our $VERSION = 'v1.5.0'; # TRIAL
+our $VERSION = 'v1.999.0'; # TRIAL
 
 use Moo;
 
 use MongoDB::Op::_Command;
 use Types::Standard qw(
     HashRef
+    Maybe
 );
 
 use namespace::clean;
@@ -33,7 +34,7 @@ use namespace::clean;
 has filter => (
     is       => 'ro',
     required => 1,
-    isa => HashRef,
+    isa => Maybe[HashRef],
 );
 
 has options => (
@@ -58,20 +59,20 @@ sub execute {
 
     my $command = [
         count => $self->coll_name,
-        query => $self->filter,
-
-        ($link->accepts_wire_version(4) ?
-            @{ $self->read_concern->as_args } : () ),
-
+        (defined $self->{filter} ? ( query => $self->{filter} ) : () ),
+        ($link->supports_read_concern ?
+            @{ $self->read_concern->as_args( $self->session ) } : () ),
         %{ $self->options },
     ];
 
     my $op = MongoDB::Op::_Command->_new(
-        db_name         => $self->db_name,
-        query           => $command,
-        query_flags     => {},
-        bson_codec      => $self->bson_codec,
-        read_preference => $self->read_preference,
+        db_name             => $self->db_name,
+        query               => $command,
+        query_flags         => {},
+        bson_codec          => $self->bson_codec,
+        read_preference     => $self->read_preference,
+        session             => $self->session,
+        monitoring_callback => $self->monitoring_callback,
     );
 
     my $res = $op->execute( $link, $topology );
