@@ -1,5 +1,4 @@
-#
-#  Copyright 2014 MongoDB, Inc.
+#  Copyright 2014 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,24 +11,25 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
+use strict;
+use warnings;
 package MongoDB::Op::_Delete;
 
 # Encapsulate a delete operation; returns a MongoDB::DeleteResult
 
 use version;
-our $VERSION = 'v1.5.0'; # TRIAL
+our $VERSION = 'v1.999.0'; # TRIAL
 
 use Moo;
 
 use MongoDB::DeleteResult;
 use MongoDB::_Protocol;
 use MongoDB::_Types qw(
+    Boolish
     Document
 );
 use Types::Standard qw(
-    Bool
     Maybe
 );
 
@@ -44,7 +44,7 @@ has filter => (
 has just_one => (
     is       => 'ro',
     required => 1,
-    isa      => Bool,
+    isa      => Boolish,
 );
 
 has collation => (
@@ -93,9 +93,10 @@ sub execute {
                 ),
                 $op_doc,
                 "MongoDB::DeleteResult",
+                "delete",
             )
         )
-        : $link->does_write_commands
+        : $link->supports_write_commands
         ? (
             $self->_send_write_command(
                 $link,
@@ -118,6 +119,7 @@ sub execute {
                 ),
                 $op_doc,
                 "MongoDB::DeleteResult",
+                "delete",
             )->assert
         )
     );
