@@ -1,5 +1,4 @@
-#
-#  Copyright 2016 MongoDB, Inc.
+#  Copyright 2016 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,19 +11,22 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
+use strict;
+use warnings;
 package MongoDB::Op::_DropCollection;
 
 # Implements a collection drop; returns a MongoDB::CommandResult
 
 use version;
-our $VERSION = 'v1.5.0'; # TRIAL
+our $VERSION = 'v1.999.0'; # TRIAL
 
 use Moo;
 
+use MongoDB::Error;
 use MongoDB::Op::_Command;
-
+use Safe::Isa;
+use Try::Tiny;
 use namespace::clean;
 
 with $_ for qw(
@@ -40,14 +42,25 @@ sub execute {
         db_name => $self->db_name,
         query   => [
             drop => $self->coll_name,
-            ( $link->accepts_wire_version(5) ? ( @{ $self->write_concern->as_args } ) : () ),
+            ( $link->supports_helper_write_concern ? ( @{ $self->write_concern->as_args } ) : () ),
         ],
-        query_flags => {},
-        bson_codec  => $self->bson_codec,
+        query_flags         => {},
+        bson_codec          => $self->bson_codec,
+        session             => $self->session,
+        monitoring_callback => $self->monitoring_callback,
     );
 
-    my $res = $op->execute($link);
-    $res->assert_no_write_concern_error;
+    my $res;
+    try {
+        $res = $op->execute($link);
+        $res->assert_no_write_concern_error;
+    }
+    catch {
+        if ( $_->$_isa("MongoDB::DatabaseError") ) {
+            return undef if $_->code == NAMESPACE_NOT_FOUND() || $_->message =~ /^ns not found/; ## no critic: make $res undef
+        }
+        die $_;
+    };
 
     return $res;
 }
