@@ -1,5 +1,4 @@
-#
-#  Copyright 2014 MongoDB, Inc.
+#  Copyright 2014 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,15 +11,16 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
+use strict;
+use warnings;
 package MongoDB::Op::_InsertOne;
 
 # Encapsulate a single-document insert operation; returns a
 # MongoDB::InsertOneResult
 
 use version;
-our $VERSION = 'v1.5.0'; # TRIAL
+our $VERSION = 'v1.999.0'; # TRIAL
 
 use Moo;
 
@@ -54,35 +54,32 @@ sub execute {
     my ( $self,     $link )       = @_;
     my ( $orig_doc, $insert_doc ) = ( $self->document );
 
-    ( $insert_doc = $self->_pre_encode_insert( $link, $orig_doc, '.' ) ),
+    ( $insert_doc = $self->_pre_encode_insert( $link->max_bson_object_size, $orig_doc, '.' ) ),
       ( $self->_set_doc_id( $insert_doc->{metadata}{_id} ) );
 
-    return ! $self->write_concern->is_acknowledged
-      ? (
-        $self->_send_legacy_op_noreply( $link,
-            MongoDB::_Protocol::write_insert( $self->full_name, $insert_doc->{bson} ),
-            $orig_doc, "MongoDB::UnacknowledgedResult" )
-      )
-      : $link->does_write_commands
-      ? (
-        $self->_send_write_command(
-            $self->_maybe_bypass(
-                $link,
-                [
-                    insert    => $self->coll_name,
-                    documents => [$insert_doc],
-                    @{ $self->write_concern->as_args },
-                ],
-            ),
-            $orig_doc,
-            "MongoDB::InsertOneResult",
-        )->assert
-      )
-      : (
-        $self->_send_legacy_op_with_gle( $link,
-            MongoDB::_Protocol::write_insert( $self->full_name, $insert_doc->{bson} ),
-            $orig_doc, "MongoDB::InsertOneResult" )->assert
-      );
+    return $self->_send_legacy_op_noreply( $link,
+        MongoDB::_Protocol::write_insert( $self->full_name, $insert_doc->{bson} ),
+        $orig_doc, "MongoDB::UnacknowledgedResult", "insert" )
+      if ! $self->write_concern->is_acknowledged;
+
+    return $self->_send_write_command(
+        $link,
+        $self->_maybe_bypass(
+            $link->supports_document_validation,
+            [
+                insert    => $self->coll_name,
+                documents => [$insert_doc],
+                @{ $self->write_concern->as_args },
+            ]
+        ),
+        $orig_doc,
+        "MongoDB::InsertOneResult"
+      )->assert
+      if $link->supports_write_commands;
+
+    return $self->_send_legacy_op_with_gle( $link,
+        MongoDB::_Protocol::write_insert( $self->full_name, $insert_doc->{bson} ),
+        $orig_doc, "MongoDB::InsertOneResult", "insert" )->assert;
 }
 
 sub _parse_cmd {
