@@ -1,5 +1,4 @@
-#
-#  Copyright 2014 MongoDB, Inc.
+#  Copyright 2014 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,14 +11,16 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
+
+use strict;
+use warnings;
 
 package MongoDB::Op::_ListIndexes;
 
 # Encapsulate index list operation; returns array ref of index documents
 
 use version;
-our $VERSION = 'v1.5.0'; # TRIAL
+our $VERSION = 'v1.999.0'; # TRIAL
 
 use Moo;
 
@@ -29,7 +30,7 @@ use MongoDB::Op::_Query;
 use MongoDB::ReadConcern;
 use MongoDB::ReadPreference;
 use Types::Standard qw(
-    InstanceOf
+  InstanceOf
 );
 use Tie::IxHash;
 use Try::Tiny;
@@ -53,7 +54,7 @@ sub execute {
     my ( $self, $link, $topology ) = @_;
 
     my $res =
-        $link->accepts_wire_version(3)
+        $link->supports_list_commands
       ? $self->_command_list_indexes( $link, $topology )
       : $self->_legacy_list_indexes( $link, $topology );
 
@@ -64,10 +65,11 @@ sub _command_list_indexes {
     my ( $self, $link, $topology ) = @_;
 
     my $op = MongoDB::Op::_Command->_new(
-        db_name         => $self->db_name,
-        query           => Tie::IxHash->new( listIndexes => $self->coll_name, cursor => {} ),
+        db_name     => $self->db_name,
+        query       => Tie::IxHash->new( listIndexes => $self->coll_name, cursor => {} ),
         query_flags => {},
-        bson_codec      => $self->bson_codec,
+        bson_codec  => $self->bson_codec,
+        monitoring_callback => $self->monitoring_callback,
     );
 
     my $res = try {
@@ -75,12 +77,14 @@ sub _command_list_indexes {
     }
     catch {
         if ( $_->$_isa("MongoDB::DatabaseError") ) {
-            return undef if $_->code == NAMESPACE_NOT_FOUND();
+            return undef if $_->code == NAMESPACE_NOT_FOUND(); ## no critic: make $res undef
         }
         die $_;
     };
 
-    return $res ? $self->_build_result_from_cursor($res) : $self->_empty_query_result($link);
+    return $res
+      ? $self->_build_result_from_cursor($res)
+      : $self->_empty_query_result($link);
 }
 
 sub _legacy_list_indexes {
@@ -88,29 +92,18 @@ sub _legacy_list_indexes {
 
     my $ns = $self->db_name . "." . $self->coll_name;
     my $op = MongoDB::Op::_Query->_new(
-        modifiers           => {},
-        allowPartialResults => 0,
-        batchSize           => 0,
-        comment             => '',
-        cursorType          => 'non_tailable',
-        limit               => 0,
-        maxAwaitTimeMS      => 0,
-        maxTimeMS           => 0,
-        noCursorTimeout     => 0,
-        oplogReplay         => 0,
-        projection          => undef,
-        skip                => 0,
-        sort                => undef,
-        db_name             => $self->db_name,
-        coll_name           => 'system.indexes',
-        full_name           => $self->db_name . '.system.indexes',
+        filter              => Tie::IxHash->new( ns => $ns ),
+        options             => MongoDB::Op::_Query->precondition_options({}),
         bson_codec          => $self->bson_codec,
         client              => $self->client,
-        read_preference     => MongoDB::ReadPreference->new,
+        coll_name           => 'system.indexes',
+        db_name             => $self->db_name,
+        full_name           => $self->db_name . '.system.indexes',
         read_concern        => MongoDB::ReadConcern->new,
-        filter              => Tie::IxHash->new( ns => $ns ),
+        read_preference     => MongoDB::ReadPreference->new,
+        monitoring_callback => $self->monitoring_callback,
     );
-  
+
     return $op->execute( $link, $topology );
 }
 
