@@ -1,5 +1,4 @@
-#
-#  Copyright 2014 MongoDB, Inc.
+#  Copyright 2014 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,14 +11,15 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
+use strict;
+use warnings;
 package MongoDB::QueryResult;
 
 # ABSTRACT: An iterator for Mongo query results
 
 use version;
-our $VERSION = 'v1.5.0'; # TRIAL
+our $VERSION = 'v1.999.0'; # TRIAL
 
 use Moo;
 use MongoDB::Error;
@@ -28,17 +28,19 @@ use MongoDB::Op::_GetMore;
 use MongoDB::Op::_KillCursors;
 use MongoDB::_Types qw(
     BSONCodec
+    ClientSession
     HostAddress
+    Intish
+    Numish
+    Stringish
 );
 use Types::Standard qw(
     Maybe
     ArrayRef
     Any
     InstanceOf
-    Int
     HashRef
-    Num
-    Str
+    Overload
 );
 use namespace::clean;
 
@@ -64,7 +66,7 @@ has _address => (
 has _full_name => (
     is       => 'ro',
     required => 1,
-    isa => Str,
+    isa => Stringish
 );
 
 has _bson_codec => (
@@ -76,12 +78,17 @@ has _bson_codec => (
 has _batch_size => (
     is       => 'ro',
     required => 1,
-    isa      => Int,
+    isa      => Intish,
 );
 
 has _max_time_ms => (
     is       => 'ro',
-    isa      => Maybe[Num],
+    isa      => Numish,
+);
+
+has _session => (
+    is => 'rwp',
+    isa => Maybe[ClientSession],
 );
 
 # attributes for tracking progress
@@ -89,7 +96,7 @@ has _max_time_ms => (
 has _cursor_at => (
     is       => 'ro',
     required => 1,
-    isa      => Num,
+    isa      => Numish,
 );
 
 sub _inc_cursor_at { $_[0]{_cursor_at}++ }
@@ -97,7 +104,7 @@ sub _inc_cursor_at { $_[0]{_cursor_at}++ }
 has _limit => (
     is       => 'ro',
     required => 1,
-    isa      => Num,
+    isa      => Numish,
 );
 
 # attributes from actual results
@@ -114,7 +121,7 @@ has _cursor_start => (
     is       => 'ro',
     required => 1,
     writer   => '_set_cursor_start',
-    isa      => Num,
+    isa      => Numish,
 );
 
 has _cursor_flags => (
@@ -127,10 +134,10 @@ has _cursor_flags => (
 has _cursor_num => (
     is       => 'ro',
     required => 1,
-    isa      => Num,
+    isa      => Numish,
 );
 
-sub _inc_cursor_num { $_[0]{_cursor_num}++ }
+sub _inc_cursor_num { $_[0]{_cursor_num} += $_[1] }
 
 has _docs => (
     is       => 'ro',
@@ -233,14 +240,16 @@ sub _get_more {
     my ($db_name, $coll_name) = split(/\./, $self->_full_name, 2);
 
     my $op = MongoDB::Op::_GetMore->_new(
-        full_name   => $self->_full_name,
-        db_name     => $db_name,
-        coll_name   => $coll_name,
-        client      => $self->_client,
-        bson_codec  => $self->_bson_codec,
-        cursor_id   => $self->_cursor_id,
-        batch_size  => $want,
-        max_time_ms => $self->_max_time_ms,
+        full_name  => $self->_full_name,
+        db_name    => $db_name,
+        coll_name  => $coll_name,
+        client     => $self->_client,
+        bson_codec => $self->_bson_codec,
+        cursor_id  => $self->_cursor_id,
+        batch_size => $want,
+        ( $self->_max_time_ms ? ( max_time_ms => $self->_max_time_ms ) : () ),
+        session             => $self->_session,
+        monitoring_callback => $self->_client->monitoring_callback,
     );
 
     my $result = $self->_client->send_direct_op( $op, $self->_address );
@@ -272,7 +281,18 @@ sub _kill_cursor {
     my ($self) = @_;
     my $cursor_id = $self->_cursor_id;
     return if !defined $cursor_id || $cursor_id == 0;
-    my $op = MongoDB::Op::_KillCursors->_new( cursor_ids => [ $cursor_id ], );
+
+    my ($db_name, $coll_name) = split(/\./, $self->_full_name, 2);
+    my $op = MongoDB::Op::_KillCursors->_new(
+        db_name             => $db_name,
+        coll_name           => $coll_name,
+        full_name           => $self->_full_name,
+        bson_codec          => $self->_bson_codec,
+        cursor_ids          => [$cursor_id],
+        client              => $self->_client,
+        session             => $self->_session,
+        monitoring_callback => $self->_client->monitoring_callback,
+    );
     $self->_client->send_direct_op( $op, $self->_address );
     $self->_set_cursor_id(0);
 }
@@ -308,7 +328,7 @@ sub DEMOLISH {
 #pod
 #pod =head2 Error handling
 #pod
-#pod Unless otherwise explictly documented, all methods throw exceptions if
+#pod Unless otherwise explicitly documented, all methods throw exceptions if
 #pod an error occurs.  The error types are documented in L<MongoDB::Error>.
 #pod
 #pod To catch and handle errors, the L<Try::Tiny> and L<Safe::Isa> modules
@@ -335,7 +355,7 @@ MongoDB::QueryResult - An iterator for Mongo query results
 
 =head1 VERSION
 
-version v1.5.0
+version v1.999.0
 
 =head1 SYNOPSIS
 
@@ -363,7 +383,7 @@ more efficient.
 
 =head2 Error handling
 
-Unless otherwise explictly documented, all methods throw exceptions if
+Unless otherwise explicitly documented, all methods throw exceptions if
 an error occurs.  The error types are documented in L<MongoDB::Error>.
 
 To catch and handle errors, the L<Try::Tiny> and L<Safe::Isa> modules
@@ -438,7 +458,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2016 by MongoDB, Inc.
+This software is Copyright (c) 2018 by MongoDB, Inc.
 
 This is free software, licensed under:
 
