@@ -1,5 +1,4 @@
-#
-#  Copyright 2014 MongoDB, Inc.
+#  Copyright 2015 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,14 +11,15 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
+use strict;
+use warnings;
 package MongoDB::ReadConcern;
 
 # ABSTRACT: Encapsulate and validate a read concern
 
 use version;
-our $VERSION = 'v1.5.0'; # TRIAL
+our $VERSION = 'v1.999.0'; # TRIAL
 
 use Moo;
 use MongoDB::Error;
@@ -56,13 +56,6 @@ has level => (
     predicate => 'has_level',
 );
 
-has _as_args => (
-    is        => 'lazy',
-    isa       => ArrayRef,
-    reader    => 'as_args',
-    builder   => '_build_as_args',
-);
-
 sub BUILD {
     my $self = shift;
     if ( defined $self->{level} ) {
@@ -70,16 +63,30 @@ sub BUILD {
     }
 }
 
-sub _build_as_args {
-    my ($self) = @_;
+# public interface for compatibility, but undocumented
+sub as_args {
+    my ( $self, $session ) = @_;
 
+    # if session is defined and operation_time is not, then either the
+    # operation_time was not sent on the response from the server for this
+    # session or the session has causal consistency disabled.
     if ( $self->{level} ) {
         return [
-            readConcern => { level => $self->{level} }
+            readConcern => {
+              level => $self->{level},
+              ( defined $session && defined $session->operation_time
+                ? ( afterClusterTime => $session->operation_time )
+                : () ),
+            }
         ];
     }
     else {
-        return [];
+        return [
+            ( defined $session && defined $session->operation_time
+              ? ( readConcern => { afterClusterTime => $session->operation_time } )
+              : ()
+            )
+        ];
     }
 }
 
@@ -97,7 +104,7 @@ MongoDB::ReadConcern - Encapsulate and validate a read concern
 
 =head1 VERSION
 
-version v1.5.0
+version v1.999.0
 
 =head1 SYNOPSIS
 
@@ -159,7 +166,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2016 by MongoDB, Inc.
+This software is Copyright (c) 2018 by MongoDB, Inc.
 
 This is free software, licensed under:
 
