@@ -1,5 +1,4 @@
-#
-#  Copyright 2014 MongoDB, Inc.
+#  Copyright 2014 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,19 +11,21 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
+use strict;
+use warnings;
 package MongoDB::ReadPreference;
 
 # ABSTRACT: Encapsulate and validate read preferences
 
 use version;
-our $VERSION = 'v1.5.0'; # TRIAL
+our $VERSION = 'v1.999.0'; # TRIAL
 
 use Moo;
 use MongoDB::Error;
 use MongoDB::_Types qw(
     ArrayOfHashRef
+    MaxStalenessNum
     NonNegNum
     ReadPrefMode
 );
@@ -73,20 +74,20 @@ has tag_sets => (
     coerce  => ArrayOfHashRef->coercion,
 );
 
-#pod =attr max_staleness_ms
+#pod =attr max_staleness_seconds
 #pod
-#pod The C<max_staleness_ms> parameter represents the maximum replication lag in
-#pod milliseconds (wall clock time) that a secondary can suffer and still be
-#pod eligible for reads. The default is zero, which disables staleness checks.
+#pod The C<max_staleness_seconds> parameter represents the maximum replication lag in
+#pod seconds (wall clock time) that a secondary can suffer and still be
+#pod eligible for reads. The default is -1, which disables staleness checks.
 #pod
-#pod If the C<mode> is 'primary', then C<max_staleness_ms> must not be supplied.
+#pod If the C<mode> is 'primary', then C<max_staleness_seconds> must not be supplied.
 #pod
 #pod =cut
 
-has max_staleness_ms => (
+has max_staleness_seconds => (
     is => 'ro',
-    isa => NonNegNum,
-    default => 0,
+    isa => MaxStalenessNum,
+    default => -1,
 );
 
 sub BUILD {
@@ -96,8 +97,8 @@ sub BUILD {
         MongoDB::UsageError->throw("A tag set list is not allowed with read preference mode 'primary'");
     }
 
-    if ( $self->mode eq 'primary' && $self->max_staleness_ms ) {
-        MongoDB::UsageError->throw("A positive max_staleness_ms is not allowed with read preference mode 'primary'");
+    if ( $self->mode eq 'primary' && $self->max_staleness_seconds > 0 ) {
+        MongoDB::UsageError->throw("A positive max_staleness_seconds is not allowed with read preference mode 'primary'");
     }
 
     return;
@@ -119,7 +120,7 @@ sub for_mongos {
     return {
         mode => $self->mode,
         ( $self->has_empty_tag_sets ? () : ( tags => $self->tag_sets ) ),
-        ( $self->max_staleness_ms > 0 ? ( maxStalenessMS => $self->max_staleness_ms ) : () ),
+        ( $self->max_staleness_seconds > 0 ? ( maxStalenessSeconds => int($self->max_staleness_seconds )) : () ),
     };
 }
 
@@ -135,8 +136,8 @@ sub as_string {
         }
         $string .= " (" . join( ",", map { "{$_}" } @ts ) . ")";
     }
-    if ( $self->max_staleness_ms > 0) {
-        $string .= " ( maxStalenessMS: " . $self->max_staleness_ms . " )";
+    if ( $self->max_staleness_seconds > 0) {
+        $string .= " ( maxStalenessSeconds: " . $self->max_staleness_seconds . " )";
     }
     return $string;
 }
@@ -156,7 +157,7 @@ MongoDB::ReadPreference - Encapsulate and validate read preferences
 
 =head1 VERSION
 
-version v1.5.0
+version v1.999.0
 
 =head1 SYNOPSIS
 
@@ -296,13 +297,13 @@ eligibility of servers, such as for data center awareness.
 The application of C<tag_sets> varies depending on the C<mode> parameter.  If
 the C<mode> is 'primary', then C<tag_sets> must not be supplied.
 
-=head2 max_staleness_ms
+=head2 max_staleness_seconds
 
-The C<max_staleness_ms> parameter represents the maximum replication lag in
-milliseconds (wall clock time) that a secondary can suffer and still be
-eligible for reads. The default is zero, which disables staleness checks.
+The C<max_staleness_seconds> parameter represents the maximum replication lag in
+seconds (wall clock time) that a secondary can suffer and still be
+eligible for reads. The default is -1, which disables staleness checks.
 
-If the C<mode> is 'primary', then C<max_staleness_ms> must not be supplied.
+If the C<mode> is 'primary', then C<max_staleness_seconds> must not be supplied.
 
 =for Pod::Coverage has_empty_tag_sets for_mongos as_string
 
@@ -334,7 +335,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2016 by MongoDB, Inc.
+This software is Copyright (c) 2018 by MongoDB, Inc.
 
 This is free software, licensed under:
 
