@@ -1,5 +1,4 @@
-#
-#  Copyright 2015 MongoDB, Inc.
+#  Copyright 2015 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,7 +11,9 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
+
+use strict;
+use warnings;
 
 package MongoDB::Role::_BypassValidation;
 
@@ -20,12 +21,12 @@ package MongoDB::Role::_BypassValidation;
 # to a command
 
 use version;
-our $VERSION = 'v1.5.0'; # TRIAL
+our $VERSION = 'v1.999.0'; # TRIAL
 
 use Moo::Role;
 
-use Types::Standard qw(
-  Bool
+use MongoDB::_Types qw(
+    Boolish
 );
 use boolean;
 
@@ -33,16 +34,16 @@ use namespace::clean;
 
 has bypassDocumentValidation => (
     is  => 'ro',
-    isa => Bool
+    isa => Boolish,
 );
 
-# args not unpacked for efficiency; args are self, link, command;
-# returns (unmodified) link and command
+# args not unpacked for efficiency; args are self, validation supported
+# flag, original command; returns (possibly modified) command
 sub _maybe_bypass {
     push @{ $_[2] },
       bypassDocumentValidation => ( $_[0]->bypassDocumentValidation ? true : false )
-      if defined $_[0]->bypassDocumentValidation && $_[1]->accepts_wire_version(4);
-    return $_[1], $_[2];
+      if $_[1] && defined $_[0]->bypassDocumentValidation;
+    return $_[2];
 }
 
 1;
