@@ -1,5 +1,4 @@
-#
-#  Copyright 2015 MongoDB, Inc.
+#  Copyright 2015 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,14 +11,15 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
+use strict;
+use warnings;
 package MongoDB::Role::_PrivateConstructor;
 
 # MongoDB interface for a private constructor
 
 use version;
-our $VERSION = 'v1.5.0'; # TRIAL
+our $VERSION = 'v1.999.0'; # TRIAL
 
 use Moo::Role;
 
@@ -32,8 +32,8 @@ use namespace::clean;
 # the private constructor blesses args directly to the class for speed.
 BEGIN {
   WITH_ASSERTS
-  ? eval 'sub _new { my $class = shift; $class->new(@_) }'
-  : eval 'sub _new { my $class = shift; return bless {@_}, $class }';
+  ? eval 'sub _new { my $class = shift; $class->new(@_) }' ## no critic
+  : eval 'sub _new { my $class = shift; return bless {@_}, $class }'; ## no critic
 }
 
 1;
