@@ -1,5 +1,4 @@
-#
-#  Copyright 2014 MongoDB, Inc.
+#  Copyright 2014 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,15 +11,16 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
+use strict;
+use warnings;
 package MongoDB::Role::_ReadPrefModifier;
 
 # MongoDB role to modify OP_QUERY query document or flags to account
 # for topology-specific treatment of a read-preference (if any)
 
 use version;
-our $VERSION = 'v1.5.0'; # TRIAL
+our $VERSION = 'v1.999.0'; # TRIAL
 
 use Moo::Role;
 
@@ -78,7 +78,7 @@ sub _apply_mongos_read_prefs {
     elsif ( $mode eq 'secondaryPreferred' ) {
         $query_flags->{slave_ok} = 1;
         $need_read_pref = 1
-          unless $read_pref->has_empty_tag_sets && $read_pref->max_staleness_ms == 0 ;
+          unless $read_pref->has_empty_tag_sets && $read_pref->max_staleness_seconds == -1;
     }
     else {
         MongoDB::InternalError->throw("invalid read preference mode '$mode'");
