@@ -1,5 +1,4 @@
-#
-#  Copyright 2014 MongoDB, Inc.
+#  Copyright 2014 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,35 +11,50 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
+
+use strict;
+use warnings;
 
 package MongoDB::_Credential;
 
 use version;
-our $VERSION = 'v1.5.0'; # TRIAL
+our $VERSION = 'v1.999.0'; # TRIAL
 
 use Moo;
 use MongoDB::Error;
 use MongoDB::Op::_Command;
 use MongoDB::_Types qw(
-    AuthMechanism
-    NonEmptyStr
+  AuthMechanism
+  NonEmptyStr
 );
 
 use Digest::MD5 qw/md5_hex/;
 use Encode qw/encode/;
 use MIME::Base64 qw/encode_base64 decode_base64/;
+use Safe::Isa;
 use Tie::IxHash;
 use Try::Tiny;
+use MongoDB::_Types qw(
+    Boolish
+);
 use Types::Standard qw(
-    Bool
-    HashRef
-    InstanceOf
-    Str
+  CodeRef
+  HashRef
+  InstanceOf
+  Maybe
+  Str
 );
 
 use namespace::clean -except => 'meta';
 
+# Required so we're sure it's passed explicitly, even if undef, so we don't
+# miss wiring it up.
+has monitoring_callback => (
+    is => 'ro',
+    required => 1,
+    isa => Maybe[CodeRef],
+);
+
 has mechanism => (
     is       => 'ro',
     isa      => AuthMechanism,
@@ -67,7 +81,7 @@ has password => (
 
 has pw_is_digest => (
     is  => 'ro',
-    isa => Bool,
+    isa => Boolish,
 );
 
 has mechanism_properties => (
@@ -82,22 +96,46 @@ has _digested_password => (
     builder => '_build__digested_password',
 );
 
-has _scram_client => (
+has _scram_sha1_client => (
     is      => 'lazy',
-    isa     => InstanceOf['Authen::SCRAM::Client'],
-    builder => '_build__scram_client',
+    isa     => InstanceOf ['Authen::SCRAM::Client'],
+    builder => '_build__scram_sha1_client',
 );
 
-sub _build__scram_client {
+has _scram_sha256_client => (
+    is      => 'lazy',
+    isa     => InstanceOf ['Authen::SCRAM::Client'],
+    builder => '_build__scram_sha256_client',
+);
+
+sub _build__scram_sha1_client {
     my ($self) = @_;
     # loaded only demand as it has a long load time relative to other
     # modules
     require Authen::SCRAM::Client;
-    Authen::SCRAM::Client->VERSION(0.003);
+    Authen::SCRAM::Client->VERSION(0.010);
     return Authen::SCRAM::Client->new(
-        username      => $self->username,
-        password      => $self->_digested_password,
-        skip_saslprep => 1,
+        username                => $self->username,
+        password                => $self->_digested_password,
+        digest                  => 'SHA-1',
+        minimum_iteration_count => 4096,
+        skip_saslprep           => 1,
+    );
+}
+
+sub _build__scram_sha256_client {
+    my ($self) = @_;
+    # loaded only demand as it has a long load time relative to other
+    # modules
+    require Authen::SCRAM::Client;
+    Authen::SCRAM::Client->VERSION(0.007);
+    require Authen::SASL::SASLprep;
+    return Authen::SCRAM::Client->new(
+        username                => $self->username,
+        password                => Authen::SASL::SASLprep::saslprep($self->password),
+        digest                  => 'SHA-256',
+        minimum_iteration_count => 4096,
+        skip_saslprep           => 1,
     );
 }
 
@@ -110,9 +148,10 @@ sub _build__digested_password {
 sub _build_source {
     my ($self) = @_;
     my $mech = $self->mechanism;
-    return $mech eq 'DEFAULT' || $mech eq 'MONGODB-CR' || $mech eq 'SCRAM-SHA-1'
-      ? 'admin'
-      : '$external';
+    return
+      $mech eq 'MONGODB-X509' || $mech eq 'PLAIN' || $mech eq 'GSSAPI'
+      ? '$external'
+      : 'admin';
 }
 
 #<<< No perltidy
@@ -124,7 +163,6 @@ my %CONSTRAINTS = (
         mechanism_properties => sub { !keys %$_ },
     },
     'MONGODB-X509' => {
-        username             => sub { length },
         password             => sub { ! length },
         source               => sub { $_ eq '$external' },
         mechanism_properties => sub { !keys %$_ },
@@ -145,6 +183,12 @@ my %CONSTRAINTS = (
         source               => sub { length },
         mechanism_properties => sub { !keys %$_ },
     },
+    'SCRAM-SHA-256' => {
+        username             => sub { length },
+        password             => sub { length },
+        source               => sub { length },
+        mechanism_properties => sub { !keys %$_ },
+    },
     'DEFAULT' => {
         username             => sub { length },
         password             => sub { length },
@@ -160,7 +204,8 @@ sub BUILD {
     my $mech = $self->mechanism;
 
     # validate attributes for given mechanism
-    while ( my ( $key, $validator ) = each %{ $CONSTRAINTS{$mech} } ) {
+    for my $key ( sort keys %{ $CONSTRAINTS{$mech} } ) {
+        my $validator = $CONSTRAINTS{$mech}{$key};
         local $_ = $self->$key;
         unless ( $validator->() ) {
             MongoDB::UsageError->throw("invalid field $key in $mech credential");
@@ -177,16 +222,16 @@ sub BUILD {
 }
 
 sub authenticate {
-    my ( $self, $link, $bson_codec ) = @_;
+    my ( $self, $server, $link, $bson_codec ) = @_;
 
     my $mech = $self->mechanism;
     if ( $mech eq 'DEFAULT' ) {
-        $mech = $link->accepts_wire_version(3) ? 'SCRAM-SHA-1' : 'MONGODB-CR';
+        $mech = $self->_get_default_mechanism($server, $link);
     }
     my $method = "_authenticate_$mech";
     $method =~ s/-/_/g;
 
-    return $self->$method($link, $bson_codec);
+    return $self->$method( $link, $bson_codec );
 }
 
 #--------------------------------------------------------------------------#
@@ -198,7 +243,8 @@ sub _authenticate_NONE () { 1 }
 sub _authenticate_MONGODB_CR {
     my ( $self, $link, $bson_codec ) = @_;
 
-    my $nonce = $self->_send_command( $link, $bson_codec, 'admin', { getnonce => 1 } )->output->{nonce};
+    my $nonce = $self->_send_command( $link, $bson_codec, 'admin', { getnonce => 1 } )
+      ->output->{nonce};
     my $key =
       md5_hex( encode( "UTF-8", $nonce . $self->username . $self->_digested_password ) );
 
@@ -216,10 +262,18 @@ sub _authenticate_MONGODB_CR {
 sub _authenticate_MONGODB_X509 {
     my ( $self, $link, $bson_codec ) = @_;
 
+    my $username = $self->username;
+
+    if ( !$username && !$link->supports_x509_user_from_cert ) {
+        $username = $link->client_certificate_subject
+          or MongoDB::UsageError->throw(
+            "Could not extract subject from client SSL certificate");
+    }
+
     my $command = Tie::IxHash->new(
         authenticate => 1,
-        user         => $self->username,
         mechanism    => "MONGODB-X509",
+        ( $username ? ( user => $username ) : () ),
     );
     $self->_send_command( $link, $bson_codec, $self->source, $command );
 
@@ -266,45 +320,59 @@ sub _authenticate_GSSAPI {
         my $step = $client->client_start;
         $self->_assert_gssapi( $client,
             "Could not start GSSAPI. Did you run kinit?  Error was: " );
-        my ( $sasl_resp, $conv_id, $done ) = $self->_sasl_start( $link, $bson_codec, $step, 'GSSAPI' );
+        my ( $sasl_resp, $conv_id, $done ) =
+          $self->_sasl_start( $link, $bson_codec, $step, 'GSSAPI' );
 
         # iterate, but with maximum number of exchanges to prevent endless loop
         for my $i ( 1 .. 10 ) {
             last if $done;
             $step = $client->client_step($sasl_resp);
             $self->_assert_gssapi( $client, "GSSAPI step error: " );
-            ( $sasl_resp, $conv_id, $done ) = $self->_sasl_continue( $link, $bson_codec, $step, $conv_id );
+            ( $sasl_resp, $conv_id, $done ) =
+              $self->_sasl_continue( $link, $bson_codec, $step, $conv_id );
         }
     }
     catch {
-        MongoDB::AuthError->throw("GSSAPI error: $_");
+        my $msg = $_->$_isa("MongoDB::Error") ? $_->message : "$_";
+        MongoDB::AuthError->throw("GSSAPI error: $msg");
     };
 
     return 1;
 }
 
 sub _authenticate_SCRAM_SHA_1 {
-    my ( $self, $link, $bson_codec ) = @_;
+    my $self = shift;
 
-    my $client = $self->_scram_client;
-
-    my ( $msg, $sasl_resp, $conv_id, $done );
-    try {
-        $msg = $client->first_msg;
-        ( $sasl_resp, $conv_id, $done ) = $self->_sasl_start( $link, $bson_codec, $msg, 'SCRAM-SHA-1' );
-        $msg = $client->final_msg($sasl_resp);
-        ( $sasl_resp, $conv_id, $done ) = $self->_sasl_continue( $link, $bson_codec, $msg, $conv_id );
-        $client->validate($sasl_resp);
-        # might require an empty payload to complete SASL conversation
-        $self->_sasl_continue( $link, $bson_codec, "", $conv_id ) if !$done;
-    }
-    catch {
-        MongoDB::AuthError->throw("SCRAM-SHA-1 error: $_");
-    };
+    $self->_scram_auth(@_, $self->_scram_sha1_client, 'SCRAM-SHA-1');
 
     return 1;
 }
 
+sub _authenticate_SCRAM_SHA_256 {
+    my $self = shift;
+
+    $self->_scram_auth(@_, $self->_scram_sha256_client, 'SCRAM-SHA-256');
+
+    return 1;
+}
+
+sub _get_default_mechanism {
+    my ( $self, $server, $link ) = @_;
+
+    if ( my $supported = $server->is_master->{saslSupportedMechs} ) {
+        if ( grep { $_ eq 'SCRAM-SHA-256' } @$supported ) {
+            return 'SCRAM-SHA-256';
+        }
+        return 'SCRAM-SHA-1';
+    }
+
+    if ( $link->supports_scram_sha1 ) {
+        return 'SCRAM-SHA-1';
+    }
+
+    return 'MONGODB-CR';
+}
+
 #--------------------------------------------------------------------------#
 # GSSAPI/SASL methods
 #--------------------------------------------------------------------------#
@@ -331,13 +399,30 @@ sub _assert_gssapi {
     return 1;
 }
 
+# PERL-801: GSSAPI broke in some cases after switching to binary
+# payloads, so fall back to base64 encoding for that mechanism.
+sub _sasl_encode_payload {
+    my ( $self, $payload ) = @_;
+    $payload = "" unless defined $payload;
+    return encode_base64( $payload, "" ) if $self->mechanism eq 'GSSAPI';
+    $payload = encode( "UTF-8", $payload );
+    return \$payload;
+}
+
+sub _sasl_decode_payload {
+    my ( $self, $payload ) = @_;
+    return "" unless defined $payload && length $payload;
+    return $payload->data if ref $payload;
+    return decode_base64($payload);
+}
+
 sub _sasl_start {
     my ( $self, $link, $bson_codec, $payload, $mechanism ) = @_;
 
     my $command = Tie::IxHash->new(
         saslStart     => 1,
         mechanism     => $mechanism,
-        payload       => $payload ? encode_base64( $payload, "" ) : "",
+        payload       => $self->_sasl_encode_payload($payload),
         autoAuthorize => 1,
     );
 
@@ -350,7 +435,7 @@ sub _sasl_continue {
     my $command = Tie::IxHash->new(
         saslContinue   => 1,
         conversationId => $conv_id,
-        payload        => $payload ? encode_base64( $payload, "" ) : "",
+        payload        => $self->_sasl_encode_payload($payload),
     );
 
     return $self->_sasl_send( $link, $bson_codec, $command );
@@ -358,24 +443,49 @@ sub _sasl_continue {
 
 sub _sasl_send {
     my ( $self, $link, $bson_codec, $command ) = @_;
-    my $output = $self->_send_command( $link, $bson_codec, $self->source, $command )->output;
+    my $output =
+      $self->_send_command( $link, $bson_codec, $self->source, $command )->output;
 
-    my $sasl_resp = $output->{payload} ? decode_base64( $output->{payload} ) : "";
-    return ( $sasl_resp, $output->{conversationId}, $output->{done} );
+    return (
+        $self->_sasl_decode_payload( $output->{payload} ),
+        $output->{conversationId},
+        $output->{done}
+    );
+}
+
+sub _scram_auth {
+    my ( $self, $link, $bson_codec, $client, $mech ) = @_;
+
+    my ( $msg, $sasl_resp, $conv_id, $done );
+    try {
+        $msg = $client->first_msg;
+        ( $sasl_resp, $conv_id, $done ) =
+          $self->_sasl_start( $link, $bson_codec, $msg, $mech );
+        $msg = $client->final_msg($sasl_resp);
+        ( $sasl_resp, $conv_id, $done ) =
+          $self->_sasl_continue( $link, $bson_codec, $msg, $conv_id );
+        $client->validate($sasl_resp);
+        # might require an empty payload to complete SASL conversation
+        $self->_sasl_continue( $link, $bson_codec, "", $conv_id ) if !$done;
+    }
+    catch {
+        my $msg = $_->$_isa("MongoDB::Error") ? $_->message : "$_";
+        MongoDB::AuthError->throw("$mech error: $msg");
+    };
 }
 
 sub _send_command {
-    my ($self, $link, $bson_codec, $db_name, $command) = @_;
+    my ( $self, $link, $bson_codec, $db_name, $command ) = @_;
 
     my $op = MongoDB::Op::_Command->_new(
-        db_name => $db_name,
-        query => $command,
-        query_flags => {},
-        bson_codec => $bson_codec,
+        db_name             => $db_name,
+        query               => $command,
+        query_flags         => {},
+        bson_codec          => $bson_codec,
+        monitoring_callback => $self->monitoring_callback,
     );
-    my $res = $op->execute( $link );
+    my $res = $op->execute($link);
     return $res;
 }
 
-
 1;
