@@ -1,5 +1,4 @@
-#
-#  Copyright 2014 MongoDB, Inc.
+#  Copyright 2014 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,7 +11,6 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
 # Some portions of this code were copied and adapted from the Perl module
 # HTTP::Tiny, which is copyright Christian Hansen, David Golden and other
@@ -25,33 +23,34 @@ use warnings;
 package MongoDB::_Link;
 
 use version;
-our $VERSION = 'v1.5.0'; # TRIAL
+our $VERSION = 'v1.999.0'; # TRIAL
 
 use Moo;
 use Errno qw[EINTR EPIPE];
 use IO::Socket qw[SOCK_STREAM];
 use Scalar::Util qw/refaddr/;
-use Socket qw/SOL_SOCKET SO_KEEPALIVE SO_RCVBUF IPPROTO_TCP TCP_NODELAY/;
+use Socket qw/SOL_SOCKET SO_KEEPALIVE SO_RCVBUF IPPROTO_TCP TCP_NODELAY AF_INET/;
 use Time::HiRes qw/time/;
 use MongoDB::Error;
 use MongoDB::_Constants;
+use MongoDB::_Protocol;
 use MongoDB::_Types qw(
+    Boolish
     HostAddress
     NonNegNum
+    Numish
     ServerDesc
 );
 use Types::Standard qw(
-    Bool
     HashRef
     Maybe
-    Num
     Str
     Undef
 );
 use namespace::clean;
 
 my $SOCKET_CLASS =
-  eval { require IO::Socket::IP; IO::Socket::IP->VERSION(0.25) }
+  eval { require IO::Socket::IP; IO::Socket::IP->VERSION(0.32) }
   ? 'IO::Socket::IP'
   : 'IO::Socket::INET';
 
@@ -64,18 +63,18 @@ has address => (
 has connect_timeout => (
     is => 'ro',
     default => 20,
-    isa => Num,
+    isa => Numish,
 );
 
 has socket_timeout => (
     is => 'ro',
     default => 30,
-    isa => Num|Undef,
+    isa => Numish|Undef,
 );
 
 has with_ssl => (
     is => 'ro',
-    isa => Bool,
+    isa => Boolish,
 );
 
 has SSL_options => (
@@ -115,18 +114,105 @@ for my $f ( @is_master_fields ) {
     );
 }
 
-# for caching wire version >= 2
-has does_write_commands => (
+# wire version >= 2
+has supports_write_commands => (
     is => 'rwp',
     init_arg => undef,
-    isa => Bool,
+    isa => Boolish,
 );
 
-# for caching wire version >= 5
+# wire version >= 3
+has supports_list_commands => (
+    is => 'rwp',
+    init_arg => undef,
+    isa => Boolish,
+);
+
+has supports_scram_sha1 => (
+    is => 'rwp',
+    init_arg => undef,
+    isa => Boolish,
+);
+
+# wire version >= 4
+has supports_document_validation => (
+    is => 'rwp',
+    init_arg => undef,
+    isa => Boolish,
+);
+
+has supports_explain_command => (
+    is => 'rwp',
+    init_arg => undef,
+    isa => Boolish,
+);
+
+has supports_query_commands => (
+    is => 'rwp',
+    init_arg => undef,
+    isa => Boolish,
+);
+
+has supports_find_modify_write_concern => (
+    is => 'rwp',
+    init_arg => undef,
+    isa => Boolish,
+);
+
+has supports_fsync_command => (
+    is => 'rwp',
+    init_arg => undef,
+    isa => Boolish,
+);
+
+has supports_read_concern => (
+    is => 'rwp',
+    init_arg => undef,
+    isa => Boolish,
+);
+
+# wire version >= 5
 has supports_collation => (
     is => 'rwp',
     init_arg => undef,
-    isa => Bool,
+    isa => Boolish,
+);
+
+has supports_helper_write_concern => (
+    is => 'rwp',
+    init_arg => undef,
+    isa => Boolish,
+);
+
+has supports_x509_user_from_cert => (
+    is => 'rwp',
+    init_arg => undef,
+    isa => Boolish,
+);
+
+# for caching wire version >=6
+has supports_arrayFilters => (
+    is => 'rwp',
+    init_arg => undef,
+    isa => Boolish,
+);
+
+has supports_clusterTime => (
+    is => 'rwp',
+    init_arg => undef,
+    isa => Boolish,
+);
+
+has supports_db_aggregation => (
+    is => 'rwp',
+    init_arg => undef,
+    isa => Boolish,
+);
+
+has supports_retryWrites => (
+    is => 'rwp',
+    init_arg => undef,
+    isa => Boolish,
 );
 
 my @connection_state_fields = qw(
@@ -165,9 +251,16 @@ sub connect {
 
     my ($host, $port) = split /:/, $self->address;
 
+    # PERL-715: For 'localhost' where MongoDB is only listening on IPv4 and
+    # getaddrinfo returns an IPv6 address before an IPv4 address, some
+    # operating systems tickle a bug in IO::Socket::IP that causes
+    # connection attempts to fail before trying the IPv4 address.  As a
+    # workaround, we always force 'localhost' to use IPv4.
+
     my $fh = $SOCKET_CLASS->new(
         PeerHost => $host,
         PeerPort => $port,
+        ( lc($host) eq 'localhost' ? ( Family => AF_INET ) : () ),
         Proto    => 'tcp',
         Type     => SOCK_STREAM,
         Timeout  => $self->connect_timeout >= 0 ? $self->connect_timeout : undef,
@@ -226,8 +319,37 @@ sub set_metadata {
     $self->_set_max_message_size_bytes( $server->is_master->{maxMessageSizeBytes}
           || 2 * $self->max_bson_object_size );
 
-    $self->_set_does_write_commands( $self->accepts_wire_version(2) );
-    $self->_set_supports_collation( $self->accepts_wire_version(5) );
+    if ( $self->accepts_wire_version(2) ) {
+        $self->_set_supports_write_commands(1);
+    }
+    if ( $self->accepts_wire_version(3) ) {
+        $self->_set_supports_list_commands(1);
+        $self->_set_supports_scram_sha1(1);
+    }
+    if ( $self->accepts_wire_version(4) ) {
+        $self->_set_supports_document_validation(1);
+        $self->_set_supports_explain_command(1);
+        $self->_set_supports_query_commands(1);
+        $self->_set_supports_find_modify_write_concern(1);
+        $self->_set_supports_fsync_command(1);
+        $self->_set_supports_read_concern(1);
+    }
+    if ( $self->accepts_wire_version(5) ) {
+        $self->_set_supports_collation(1);
+        $self->_set_supports_helper_write_concern(1);
+        $self->_set_supports_x509_user_from_cert(1);
+    }
+    if ( $self->accepts_wire_version(6) ) {
+        $self->_set_supports_arrayFilters(1);
+        $self->_set_supports_clusterTime(1);
+        $self->_set_supports_db_aggregation(1);
+        $self->_set_supports_retryWrites(
+            defined( $server->logical_session_timeout_minutes )
+              && ( $server->type ne 'Standalone' )
+            ? 1
+            : 0
+        );
+    }
 
     return;
 }
@@ -259,6 +381,22 @@ sub start_ssl {
     }
 }
 
+sub client_certificate_subject {
+    my ($self) = @_;
+    return "" unless $self->fh && $self->fh->isa("IO::Socket::SSL");
+
+    my $client_cert = $self->fh->sock_certificate()
+      or return "";
+
+    my $subject_raw = Net::SSLeay::X509_get_subject_name($client_cert)
+      or return "";
+
+    my $subject =
+      Net::SSLeay::X509_NAME_print_ex( $subject_raw, Net::SSLeay::XN_FLAG_RFC2253() );
+
+    return $subject;
+}
+
 sub close {
     my ($self) = @_;
     $self->_close
@@ -283,7 +421,19 @@ sub is_connected {
 }
 
 sub write {
-    my ( $self, $buf ) = @_;
+    my ( $self, $buf, $write_opt ) = @_;
+    $write_opt ||= {};
+
+    if (
+        !$write_opt->{disable_compression}
+        && $self->server
+        && $self->server->compressor
+    ) {
+        $buf = MongoDB::_Protocol::compress(
+            $buf,
+            $self->server->compressor,
+        );
+    }
 
     my ( $len, $off, $pending, $nfound, $r ) = ( length($buf), 0 );
 
@@ -403,6 +553,7 @@ sub read {
         }
 
         if ( !defined $len ) {
+            next if length($msg) < 4;
             $len = unpack( P_INT32, $msg );
             MongoDB::ProtocolError->throw(
                 qq/Server reply of size $len exceeds maximum of / . $self->{max_message_size_bytes} )
