@@ -1,5 +1,4 @@
-#
-#  Copyright 2014 MongoDB, Inc.
+#  Copyright 2014 - present MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,22 +11,28 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
+use strict;
+use warnings;
 package MongoDB::_Types;
 
 # MongoDB type definitions
 
 use version;
-our $VERSION = 'v1.5.0'; # TRIAL
+our $VERSION = 'v1.999.0'; # TRIAL
 
 use Type::Library
   -base,
   -declare => qw(
   ArrayOfHashRef
   AuthMechanism
+  Boolish
   Booleanpm
   BSONCodec
+  BSONDoc
+  ClientSession
+  CompressionType
+  ZlibCompressionLevel
   ConnectType
   CursorType
   DBRefColl
@@ -38,13 +43,18 @@ use Type::Library
   HeartbeatFreq
   HostAddress
   HostAddressList
+  Intish
   IndexModel
   IndexModelList
   IxHash
+  MaxStalenessNum
+  MaybeHashRef
   MongoDBCollection
   MongoDBDatabase
+  BSONTimestamp
   NonEmptyStr
   NonNegNum
+  Numish
   OID
   OrderedDoc
   PairArrayRef
@@ -55,6 +65,7 @@ use Type::Library
   ServerType
   SingleChar
   SingleKeyHash
+  Stringish
   TopologyType
   WriteConcern
 );
@@ -62,12 +73,15 @@ use Type::Library
 use Type::Utils -all;
 use Types::Standard qw(
     Any
+    Bool
     ArrayRef
     Dict
     HashRef
+    Int
     Maybe
     Num
     Optional
+    Overload
     Ref
     Str
     Undef
@@ -81,20 +95,38 @@ require Tie::IxHash;
 # Type declarations (without inherited coercions)
 #--------------------------------------------------------------------------#
 
+declare Stringish, as Str|Overload['""'];
+
+declare Numish, as Num|Overload['0+'];
+
+# Types::Standard::Bool is overly restrictive, not allowing objects that
+# overload boolification, and Overload['bool'] doesn't detect objects that
+# overload via fallback, so we use this type for documentation purposes,
+# but allow any actual type.
+declare Boolish, as Any;
+
 declare ArrayOfHashRef, as ArrayRef [HashRef];
 
 enum AuthMechanism,
-  [qw/NONE DEFAULT MONGODB-CR MONGODB-X509 GSSAPI PLAIN SCRAM-SHA-1/];
-
-class_type Booleanpm, { class => 'boolean' };
+  [qw/NONE DEFAULT MONGODB-CR MONGODB-X509 GSSAPI PLAIN SCRAM-SHA-1 SCRAM-SHA-256/];
 
 duck_type BSONCodec, [ qw/encode_one decode_one/ ];
 
+class_type BSONDoc, { class => 'BSON::Doc' };
+
+class_type ClientSession, { class => 'MongoDB::ClientSession' };
+
+enum CompressionType, [qw/zlib/];
+
+declare ZlibCompressionLevel, as Int,
+  where { $_ >= -1 && $_ <= 9 },
+  message { "zlib compression value must be value from -1 to 9" };
+
 enum ConnectType, [qw/replicaSet direct none/];
 
 enum CursorType, [qw/non_tailable tailable tailable_await/];
 
-declare ErrorStr, as Str, where { $_ }; # needs a true value
+declare ErrorStr, as Stringish, where { defined($_) && length($_) }; # needs a true value
 
 declare HashLike, as Ref, where { reftype($_) eq 'HASH' };
 
@@ -104,7 +136,7 @@ declare HeartbeatFreq, as Num,
 
 # XXX loose address validation for now.  Host part should really be hostname or
 # IPv4/IPv6 literals
-declare HostAddress, as Str,
+declare HostAddress, as Stringish,
   where { $_ =~ /^[^:]+:[0-9]+$/ and lc($_) eq $_ }, message {
     "Address '$_' either not lowercased or not formatted as 'hostname:port'"
   };
@@ -113,6 +145,8 @@ declare HostAddressList, as ArrayRef [HostAddress], message {
     "Address list <@$_> not all formatted as lowercased 'hostname:port' pairs"
 };
 
+declare Intish, as Numish, where { defined $_ and $_ == int($_) };
+
 class_type IxHash, { class => 'Tie::IxHash' };
 
 declare MaybeHashRef, as Maybe[ HashRef ];
@@ -121,12 +155,18 @@ class_type MongoDBCollection, { class => 'MongoDB::Collection' };
 
 class_type MongoDBDatabase, { class => 'MongoDB::Database' };
 
-declare NonEmptyStr, as Str, where { defined $_ && length $_ };
+class_type BSONTimestamp, { class => 'BSON::Timestamp' };
 
-declare NonNegNum, as Num,
+declare NonEmptyStr, as Stringish, where { defined $_ && length $_ };
+
+declare NonNegNum, as Numish,
   where { defined($_) && $_ >= 0 },
   message { "value must be a non-negative number" };
 
+declare MaxStalenessNum, as Numish,
+  where { defined($_) && ( $_ > 0 || $_ == -1 ) },
+  message { "value must be a positive number or -1" };
+
 declare OID, as Str, where { /\A[0-9a-f]{24}\z/ }, message {
     "Value '$_' is not a valid OID"
 };
@@ -153,13 +193,13 @@ declare SingleChar, as Str, where { length $_ eq 1 };
 declare SingleKeyHash, as HashRef, where { 1 == scalar keys %$_ };
 
 enum TopologyType,
-  [qw/Single ReplicaSetNoPrimary ReplicaSetWithPrimary Sharded Unknown/];
+  [qw/Single ReplicaSetNoPrimary ReplicaSetWithPrimary Sharded Direct Unknown/];
 
 class_type WriteConcern, { class => 'MongoDB::WriteConcern' };
 
 # after SingleKeyHash, PairArrayRef and IxHash
-declare OrderedDoc, as PairArrayRef|IxHash|SingleKeyHash;
-declare Document, as HashRef|PairArrayRef|IxHash|HashLike;
+declare OrderedDoc, as BSONDoc|PairArrayRef|IxHash|SingleKeyHash;
+declare Document, as HashRef|BSONDoc|PairArrayRef|IxHash|HashLike;
 
 # after NonEmptyStr
 declare DBRefColl, as NonEmptyStr;
@@ -176,9 +216,9 @@ declare IndexModelList, as ArrayRef [IndexModel];
 coerce ArrayOfHashRef, from HashRef, via { [$_] };
 
 coerce BSONCodec, from HashRef,
-  via { require MongoDB::BSON; MongoDB::BSON->new($_) };
+  via { require BSON; BSON->new($_) };
 
-coerce Booleanpm, from Any, via { boolean($_) };
+coerce Boolish, from Any, via { !!$_ };
 
 coerce DBRefColl, from MongoDBCollection, via { $_->name };
 
@@ -194,6 +234,8 @@ coerce IxHash, from ArrayRef, via { Tie::IxHash->new(@$_) };
 
 coerce IxHash, from HashLike, via { Tie::IxHash->new(%$_) };
 
+coerce IxHash, from BSONDoc, via { Tie::IxHash->new(@$_) };
+
 coerce OID, from Str, via { lc $_ };
 
 coerce ReadPreference, from HashRef,
